#!/usr/bin/perl
#
# SmoothWall CGIs
#
# This code is distributed under the terms of the GPL
#
# (c) The SmoothWall Team
#
# Copyright (C) 01-02-2002 Graham Smith <grhm@grhm.co.uk>
#              - Fixed DHCP Leases added
#
# $Id: dhcp.cgi,v 1.14.2.99 2007/08/16 22:50:57 franck78 Exp $
#
#  Franck	-rewrite for two or more interface
#  nov/2004	-check range is in correct subnet
#		-add NTP option
#		-add display sorting of actives leases
#  dec/2004	-add comment field to fixed leases
# 
# to do : choose a correct format for displaying dates
#	: check 'new option' against its definition

use strict;

# enable only the following on debugging purpose
#use warnings;
#use CGI::Carp 'fatalsToBrowser';

require 'CONFIG_ROOT/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";
#workaround to suppress a warning when a variable is used only once
my @dummy = ( ${Header::colouryellow} );
undef (@dummy);

our %dhcpsettings=();
our %netsettings=();
my %mainsettings=();
my %timesettings=();
my $setting = "${General::swroot}/dhcp/settings";
our $filename1 = "${General::swroot}/dhcp/advoptions"; 	# Field separator is TAB in this file (comma is standart)
							# because we need commas in the some data
our $filename2 = "${General::swroot}/dhcp/fixleases";
our $filename3 = "${General::swroot}/dhcp/advoptions-list"; # List of options+syntax
our $filename4 = "${General::swroot}/dhcp/parameters-list"; # List of allowed 'parameters' ; no checking on parameters
my $errormessage = '';
my $warnmessage = '';
my $warnNTPmessage = '';
my @nosaved=();

our $OptionTypes = 'boolean|((un)?signed )?integer (8|16|32)|ip-address|text|string|encapsulate \w+|'
		    .'array of (boolean|((un)?signed )?integer (8|16|32)|ip-address)|'
		    .'{(,| |boolean|((un)?signed )?integer (8|16|32)|ip-address|text|string)+}|'
		    .'array of {(,| |boolean|((un)?signed )?integer (8|16|32)|ip-address)+}';

&Header::showhttpheaders();
our @ITFs=('GREEN','BLUE');

#Settings1 for the first screen box
foreach my $itf (@ITFs) {
    $dhcpsettings{"ENABLE_${itf}"} = 'off';
    $dhcpsettings{"ENABLEBOOTP_${itf}"} = 'off';
    $dhcpsettings{"START_ADDR_${itf}"} = '';
    $dhcpsettings{"END_ADDR_${itf}"} = '';
    $dhcpsettings{"BASE_IP_FIXED_LEASE_${itf}"} = '';
    $dhcpsettings{"DOMAIN_NAME_${itf}"} = '';
    $dhcpsettings{"DEFAULT_LEASE_TIME_${itf}"} = '';
    $dhcpsettings{"MAX_LEASE_TIME_${itf}"} = '';
    $dhcpsettings{"WINS1_${itf}"} = '';
    $dhcpsettings{"WINS2_${itf}"} = '';
    $dhcpsettings{"DNS1_${itf}"} = '';
    $dhcpsettings{"DNS2_${itf}"} = '';
    $dhcpsettings{"NTP1_${itf}"} = '';
    $dhcpsettings{"NTP2_${itf}"} = '';
}

$dhcpsettings{'SORT_FLEASELIST'} = 'FIPADDR';
$dhcpsettings{'SORT_LEASELIST'} = 'IPADDR';

#Settings2 for editing the multi-line list
#Must not be saved with writehash !
$dhcpsettings{'FIX_MAC'} = '';
$dhcpsettings{'FIX_ADDR'} = '';
$dhcpsettings{'FIX_ENABLED'} = 'off';
$dhcpsettings{'FIX_NEXTADDR'} = '';
$dhcpsettings{'FIX_FILENAME'} = '';
$dhcpsettings{'FIX_ROOTPATH'} = '';
$dhcpsettings{'FIX_REMARK'} = '';
$dhcpsettings{'FIX_HOSTNAME'} = '';
$dhcpsettings{'FIX_ROUTER'} = '';
$dhcpsettings{'FIX_DNS'} = '';
$dhcpsettings{'ACTION'} = '';
$dhcpsettings{'KEY1'} = '';
$dhcpsettings{'KEY2'} = '';
@nosaved=('FIX_MAC','FIX_ADDR','FIX_ENABLED','FIX_NEXTADDR','FIX_FILENAME',
	    'FIX_ROOTPATH','FIX_REMARK','FIX_HOSTNAME','FIX_ROUTER','FIX_DNS');

$dhcpsettings{'ADVOPT_ENABLED'} = '';
$dhcpsettings{'ADVOPT_NAME'} = '';
$dhcpsettings{'ADVOPT_DATA'} = '';
$dhcpsettings{'SUBMIT'} = '';
$dhcpsettings{'ACTION_ALL'} = '';
unshift (@nosaved,'ADVOPT_ENABLED','ADVOPT_NAME','ADVOPT_DATA','SUBMIT','ACTION_ALL');
foreach my $itf (@ITFs) {
    $dhcpsettings{"ADVOPT_SCOPE_${itf}"} = 'off';
    unshift (@nosaved, "ADVOPT_SCOPE_${itf}");
}

# Read Ipcop settings
&General::readhash("${General::swroot}/ethernet/settings", \%netsettings);
&General::readhash("${General::swroot}/main/settings", \%mainsettings);
&General::readhash("${General::swroot}/time/settings", \%timesettings);

#Get GUI values
&Header::getcgihash(\%dhcpsettings);

open(FILE, "$filename1") or die 'Unable to open dhcp advanced options file.';
our @current1 = <FILE>;
close(FILE);
# Extract OptionDefinition
foreach my $line (@current1) {
    #chomp($line);   # remove newline        #don't know why, but this remove newline in @current1 .... !
    my @temp = split(/\t/,$line);
    AddNewOptionDefinition ($temp[1] . ' ' . $temp[2]);
}

open(FILE, "$filename2") or die 'Unable to open fixed leases file.';
our @current2 = <FILE>;
close(FILE);

# Check Settings1 first because they are needed by &buildconf
if ($dhcpsettings{'ACTION'} eq $Lang::tr{'save'}) {
    foreach my $itf (@ITFs) {
	if ($dhcpsettings{"ENABLE_${itf}"} eq 'on' ) {
	    # "Start" is defined, need "End" and vice versa
	    if ($dhcpsettings{"START_ADDR_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"START_ADDR_${itf}"}))) {
		    $errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid start address'};
		    goto ERROR;
		}
		if (!$dhcpsettings{"END_ADDR_${itf}"}) {
		    $errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid end address'};
		    goto ERROR;
		}
		if (! &General::IpInSubnet ( $dhcpsettings{"START_ADDR_${itf}"}, 
				    $netsettings{"${itf}_NETADDRESS"},
				    $netsettings{"${itf}_NETMASK"})) {
		    $errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid start address'};
		    goto ERROR;
		}
	    }
	    
	    if ($dhcpsettings{"END_ADDR_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"END_ADDR_${itf}"}))) {
		    $errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid end address'};
		    goto ERROR;
		}
		if (!$dhcpsettings{"START_ADDR_${itf}"}) {
		    $errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid start address'};
		    goto ERROR;
		}
		if (! &General::IpInSubnet ( $dhcpsettings{"END_ADDR_${itf}"}, 
				    $netsettings{"${itf}_NETADDRESS"},
				    $netsettings{"${itf}_NETMASK"})) { 
		    $errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid end address'};
		    goto ERROR;
		}
		#swap if necessary! (support 255.255.0.0 range, I doubt we need more) GE
		my @startoct = split (/\./, $dhcpsettings{"START_ADDR_${itf}"});
		my @endoct   = split (/\./, $dhcpsettings{"END_ADDR_${itf}"});
		if ( $endoct[2]*256+$endoct[3] < $startoct[2]*256+$startoct[3] ) {
		    ($dhcpsettings{"START_ADDR_${itf}"},$dhcpsettings{"END_ADDR_${itf}"}) =
			($dhcpsettings{"END_ADDR_${itf}"},$dhcpsettings{"START_ADDR_${itf}"});
		}
	    }

	    if (!($dhcpsettings{"DEFAULT_LEASE_TIME_${itf}"} =~ /^\d+$/)) {
		$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid default lease time'} . $dhcpsettings{'DEFAULT_LEASE_TIME_${itf}'};
		goto ERROR;
	    }

	    if (!($dhcpsettings{"MAX_LEASE_TIME_${itf}"} =~ /^\d+$/)) {
		$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid max lease time'} . $dhcpsettings{'MAX_LEASE_TIME_${itf}'};
		goto ERROR;
	    }

	    if ($dhcpsettings{"BASE_IP_FIXED_LEASE_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"BASE_IP_FIXED_LEASE_${itf}"}))) {
		    $errormessage = "DHCP on ${itf}: " .$Lang::tr{'dhcp base ip fixed lease'}. ' '. $Lang::tr{'invalid ip'} ;
		    goto ERROR;
		}
	    }

	    if ($dhcpsettings{"DNS1_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"DNS1_${itf}"}))) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid primary dns'};
			goto ERROR;
		}
	    }
	    if ($dhcpsettings{"DNS2_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"DNS2_${itf}"}))) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid secondary dns'};
			goto ERROR;
		}
		if (! $dhcpsettings{"DNS1_${itf}"}) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'cannot specify secondary dns without specifying primary'}; 
			goto ERROR;
		}
	    }

	    if ($dhcpsettings{"WINS1_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"WINS1_${itf}"}))) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid wins address'};
			goto ERROR;
		}
	    }
	    if ($dhcpsettings{"WINS2_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"WINS2_${itf}"})))	{
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid wins address'};
			goto ERROR;
		}
		if (! $dhcpsettings{"WINS1_${itf}"} ) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'cannot specify secondary wins without specifying primary'};
			goto ERROR;
		}		
	    }

	    if ($dhcpsettings{"NTP1_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"NTP1_${itf}"}))) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid primary ntp'};
			goto ERROR;
		}
		if ($dhcpsettings{"NTP1_${itf}"} eq $netsettings{"${itf}_ADDRESS"} && ($timesettings{'ENABLECLNTP'} ne 'on')) {
			$warnNTPmessage = "DHCP on ${itf}: " . $Lang::tr{'local ntp server specified but not enabled'};
			#goto ERROR;
		}
	    }
	    if ($dhcpsettings{"NTP2_${itf}"}) {
		if (!(&General::validip($dhcpsettings{"NTP2_${itf}"}))) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'invalid secondary ntp'};
			goto ERROR;
		}
		if ($dhcpsettings{"NTP2_${itf}"} eq $netsettings{"${itf}_ADDRESS"} && ($timesettings{'ENABLECLNTP'} ne 'on')) {
			$warnNTPmessage = "DHCP on ${itf}: " . $Lang::tr{'local ntp server specified but not enabled'};
			#goto ERROR;
		}
		if (! $dhcpsettings{"NTP1_${itf}"}) {
			$errormessage = "DHCP on ${itf}: " . $Lang::tr{'cannot specify secondary ntp without specifying primary'};
			goto ERROR;
		}
	    }
	} # enabled
    }#loop interface verify

    map (delete ($dhcpsettings{$_}) ,@nosaved,'ACTION','KEY1','KEY2');	# Must not be saved 
    &General::writehash($setting, \%dhcpsettings);		# Save good settings
    $dhcpsettings{'ACTION'} = $Lang::tr{'save'};		# create an 'ACTION'
    map ($dhcpsettings{$_} = '',@nosaved,'KEY1','KEY2');	# and reinit vars to empty
    &buildconf;
    ERROR:   							# Leave the faulty field untouched
} else {
    &General::readhash($setting, \%dhcpsettings);   		# Get saved settings and reset to good if needed
}

## Sorting of fixed leases
if ($ENV{'QUERY_STRING'} =~ /^FETHER|^FIPADDR/ ) {
    my $newsort=$ENV{'QUERY_STRING'};
    my $act=$dhcpsettings{'SORT_FLEASELIST'};
    #Reverse actual sort ?
    if ($act =~ $newsort) {
	my $Rev='';
	if ($act !~ 'Rev') {
	    $Rev='Rev';
	}
	$newsort.=$Rev;
    }
    $dhcpsettings{'SORT_FLEASELIST'}=$newsort;
    map (delete ($dhcpsettings{$_}) ,@nosaved,'ACTION','KEY1','KEY2');	# Must never be saved 
    &General::writehash($setting, \%dhcpsettings);
    &sortcurrent2;
    $dhcpsettings{'ACTION'} = 'SORT';			# create an 'ACTION'
    map ($dhcpsettings{$_} = '',@nosaved,'KEY1','KEY2');# and reinit vars to empty 
}

#Sorting of allocated leases
&Header::CheckSortOrder;


## Now manipulate the two multi-line list with Settings2. 
#  '1' suffix is for ADVANCED OPTIONS
#  '2' suffix is for FIXED LEASES

# Toggle enable/disable field on specified options.

if ($dhcpsettings{'ACTION'} eq $Lang::tr{'toggle enable disable'}.'1') {
    #move out new line
    chomp(@current1[$dhcpsettings{'KEY1'}]);
    my @temp = split(/\t/,@current1[$dhcpsettings{'KEY1'}]);		#use TAB separator !
    $temp[0] = $temp[0] eq 'on' ? '' : 'on';    # Toggle the field
    @current1[$dhcpsettings{'KEY1'}] = join ("\t",@temp)."\n";
    $dhcpsettings{'KEY1'} = ''; # End edit mode
    &General::log($Lang::tr{'dhcp advopt modified'});
    open(FILE, ">$filename1") or die 'Unable to open dhcp advanced options file.';
    print FILE @current1;
    close(FILE);
	
    #Write changes to dhcpd.conf.
    &buildconf;
}


if ($dhcpsettings{'ACTION'} eq $Lang::tr{'add'}.'1' &&
	$dhcpsettings{'SUBMIT'} ne $Lang::tr{'dhcp advopt help'}) {

    #If input box ADVOPT_NAME is empty, get value from the select;
    $dhcpsettings{'ADVOPT_NAME'} = $dhcpsettings{'ADVOPT_LIST'} if ( ! $dhcpsettings{'ADVOPT_NAME'} );

    $dhcpsettings{'ADVOPT_NAME'} =~ s/[^ \w-]//g;  	# prevent execution of code by removing everything except letters/space
    $dhcpsettings{'ADVOPT_DATA'} =~ s/`//g;		# back tik ` ? not allowed !

    if ($dhcpsettings{'ADVOPT_DATA'} eq '') {
	$errormessage=$Lang::tr{'dhcp advopt blank value'};
    }

    # Test for a new option definition string (join field name & data)
    if (ExistNewOptionDefinition ($dhcpsettings{'ADVOPT_NAME'} . ' ' . $dhcpsettings{'ADVOPT_DATA'})) {
	#only edit permitted if option definition exists
	$errormessage = $Lang::tr{'dhcp advopt definition exists'} if ($dhcpsettings{'KEY1'} eq '');
	$dhcpsettings{'ADVOPT_ENABLED'} = 'on';			# force active
	map ($dhcpsettings{"ADVOPT_SCOPE_$_"} = 'off', @ITFs);	# force global
    } elsif (AddNewOptionDefinition ($dhcpsettings{'ADVOPT_NAME'} . ' ' . $dhcpsettings{'ADVOPT_DATA'})) {
	#was a new option definition
	$dhcpsettings{'ADVOPT_ENABLED'} = 'on';			# force active
	map ($dhcpsettings{"ADVOPT_SCOPE_$_"} = 'off', @ITFs);	# force global
    } elsif (ValidNewOption ($dhcpsettings{'ADVOPT_NAME'} . ' ' . $dhcpsettings{'ADVOPT_DATA'})) {
	#was a new option, only editing or adding one instance is allowed
	if ($dhcpsettings{'KEY1'} eq '') {
	    foreach my $line (@current1) {
		#chomp($line);   # remove newline        #don't know why, but this remove newline in @current1 .... !
		my @temp = split(/\t/,$line);
		#I know, the message is not perfectly exact.
		$errormessage = $Lang::tr{'dhcp advopt definition exists'} 
			if (($dhcpsettings{'ADVOPT_NAME'} eq $temp[1]) && ($temp[2] !~ /code \d+=/));
	    }
	}
    } elsif (! `/bin/grep "^option $dhcpsettings{'ADVOPT_NAME'} " $filename3` &&
             ! `/bin/grep "^$dhcpsettings{'ADVOPT_NAME'}" $filename4`) {
	$warnmessage=$Lang::tr{'dhcp advopt unknown'}.': '.$dhcpsettings{'ADVOPT_NAME'};
    }
    unless ($errormessage) {

	my $scope = '';
	foreach my $itf (@ITFs) {  # buils "RED,GREEN,ORANGE,... based on selection
	    $scope .= $dhcpsettings{"ADVOPT_SCOPE_${itf}"} eq 'on' ? "\t$itf" : "\toff" ;
	}
	if ($dhcpsettings{'KEY1'} eq '') { #add or edit ?  TAB separator !
	    unshift (@current1, "$dhcpsettings{'ADVOPT_ENABLED'}\t$dhcpsettings{'ADVOPT_NAME'}\t$dhcpsettings{'ADVOPT_DATA'}$scope\n");
	    &General::log($Lang::tr{'dhcp advopt added'});
	} else {
	    @current1[$dhcpsettings{'KEY1'}] = "$dhcpsettings{'ADVOPT_ENABLED'}\t$dhcpsettings{'ADVOPT_NAME'}\t$dhcpsettings{'ADVOPT_DATA'}$scope\n";
	    $dhcpsettings{'KEY1'} = '';       # End edit mode
	    &General::log($Lang::tr{'dhcp advopt modified'});
	}

        #Write changes to dhcpd.conf.
        &sortcurrent1;    # sort newly added/modified entry
        &buildconf;       # before calling buildconf which use fixed lease file !
    }
}

if ($dhcpsettings{'ACTION'} eq $Lang::tr{'edit'}.'1') {
    #move out new line
    my $line = @current1[$dhcpsettings{'KEY1'}];
    chomp($line);
    my @temp = split(/\t/, $line);
    $dhcpsettings{'ADVOPT_ENABLED'}=$temp[0];
    $dhcpsettings{'ADVOPT_NAME'}=$temp[1];
    $dhcpsettings{'ADVOPT_DATA'}=$temp[2];

    # read next fields which are the name (color) of an interface if this interface is scoped
    for (my $key=0; $key<@ITFs; $key++) {
	my $itf = $temp[3+$key];
	if ($itf ne 'off') # Only is an interface name is read
	{
	    $dhcpsettings{"ADVOPT_SCOPE_${itf}"} = 'on';
	}    
    }
}

if ($dhcpsettings{'ACTION'} eq $Lang::tr{'remove'}.'1') {
    splice (@current1,$dhcpsettings{'KEY1'},1);
    open(FILE, ">$filename1") or die 'Unable to open dhcp advanced options file.';
    print FILE @current1;
    close(FILE);
    $dhcpsettings{'KEY1'} = '';				# End remove mode
    &General::log($Lang::tr{'dhcp advopt removed'});
    #Write changes to dhcpd.conf.
    &buildconf;
}
#end KEY1


# Toggle enable/disable field on specified lease.
if ($dhcpsettings{'ACTION'} eq $Lang::tr{'toggle enable disable'}.'2') {
    #move out new line
    chomp(@current2[$dhcpsettings{'KEY2'}]);
    my ($FIX_MAC,$FIX_ADDR,$FIX_ENABLED,$DATA) = split(',', @current2[$dhcpsettings{'KEY2'}],4);
    $FIX_ENABLED = $FIX_ENABLED eq 'on' ? '' : 'on';    # Toggle the field
    @current2[$dhcpsettings{'KEY2'}] = "$FIX_MAC,$FIX_ADDR,$FIX_ENABLED,$DATA\n";
    $dhcpsettings{'KEY2'} = ''; # End edit mode
    &General::log($Lang::tr{'fixed ip lease modified'});
    open(FILE, ">$filename2") or die 'Unable to open fixed leases file.';
    print FILE @current2;
    close(FILE);

    #Write changes to dhcpd.conf.
    &buildconf;
}

if ($dhcpsettings{'ACTION'} eq $Lang::tr{'add'}.'2') {

    $errormessage = $Lang::tr{'dhcp fixed lease err1'}
	if ( ($dhcpsettings{'FIX_HOSTNAME'} eq '') && ($dhcpsettings{'FIX_MAC'} eq ''));
    $errormessage = myvalidiporfqdn ($dhcpsettings{'FIX_ADDR'}, $errormessage);
    $errormessage = myvalidiporfqdn ($dhcpsettings{'FIX_NEXTADDR'}, $errormessage)
	if($dhcpsettings{'FIX_NEXTADDR'});
    $errormessage = myvalidiporfqdn ($dhcpsettings{'FIX_ROUTER'}, $errormessage)
	if($dhcpsettings{'FIX_ROUTER'});
    $errormessage = myvalidiporfqdn ($dhcpsettings{'FIX_DNS'}, $errormessage)
	if($dhcpsettings{'FIX_DNS'});

    my $key = 0;
    if ($dhcpsettings{'FIX_MAC'} ne '' ) {
	$dhcpsettings{'FIX_MAC'} =~ tr/-/:/;
	$errormessage = $Lang::tr{'invalid fixed mac address'} if ( ! &General::validmac($dhcpsettings{'FIX_MAC'}));

        CHECK:foreach my $line (@current2) {
    	    my @temp = split(',', $line);
    	    if($dhcpsettings{'KEY2'} ne $key) {
 		# same MAC is OK on different subnets. This test is not complete because
		# if IP are not inside a known subnet, I don't warn.
		# Also it may be needed to put duplicate fixed lease in their right subnet definition..
		foreach my $itf (@ITFs) {
		    my $scoped = &General::IpInSubnet($dhcpsettings{'FIX_ADDR'},
						  $netsettings{"${itf}_NETADDRESS"}, 
						  $netsettings{"${itf}_NETMASK"}) &&
						  $dhcpsettings{"ENABLE_${itf}"} eq 'on';
		    if ( $scoped &&
			(lc($dhcpsettings{'FIX_MAC'}) eq lc($temp[0])) &&
			&General::IpInSubnet($temp[1],
					 $netsettings{"${itf}_NETADDRESS"}, 
					 $netsettings{"${itf}_NETMASK"})) {
			$errormessage = "$Lang::tr{'mac address in use'} $dhcpsettings{'FIX_MAC'}";
			last CHECK;
		    }
		}
	    }
	    $key++;
	}
    }

    unless ($errormessage) {
	$dhcpsettings{'FIX_REMARK'} = &Header::cleanhtml($dhcpsettings{'FIX_REMARK'});
	$dhcpsettings{'FIX_NEXTADDR'} = &Header::cleanhtml($dhcpsettings{'FIX_NEXTADDR'});
	$dhcpsettings{'FIX_HOSTNAME'} = &Header::cleanhtml($dhcpsettings{'FIX_HOSTNAME'});
	$dhcpsettings{'FIX_ROUTER'} = &Header::cleanhtml($dhcpsettings{'FIX_ROUTER'});
	$dhcpsettings{'FIX_DNS'} = &Header::cleanhtml($dhcpsettings{'FIX_DNS'});
	# remove some character unallowed in a filename, replace the comma with &comma;
	$dhcpsettings{'FIX_FILENAME'} =~ s/[<">?|*;&`]//g;		#`
	$dhcpsettings{'FIX_FILENAME'} =~ s/,/&comma;/g;
	$dhcpsettings{'FIX_ROOTPATH'} =~ s/[<">?|*;&`]//g;		#`
	$dhcpsettings{'FIX_ROOTPATH'} =~ s/,/&comma;/g;

	if ($dhcpsettings{'KEY2'} eq '') { #add or edit ?
	    unshift (@current2, "$dhcpsettings{'FIX_MAC'},$dhcpsettings{'FIX_ADDR'},$dhcpsettings{'FIX_ENABLED'},".
		"$dhcpsettings{'FIX_NEXTADDR'},$dhcpsettings{'FIX_FILENAME'},$dhcpsettings{'FIX_ROOTPATH'},".
		"$dhcpsettings{'FIX_REMARK'},$dhcpsettings{'FIX_HOSTNAME'},$dhcpsettings{'FIX_ROUTER'},".
		"$dhcpsettings{'FIX_DNS'}\n");
	    &General::log($Lang::tr{'fixed ip lease added'});
	} else {
	    @current2[$dhcpsettings{'KEY2'}] = "$dhcpsettings{'FIX_MAC'},$dhcpsettings{'FIX_ADDR'},".
		"$dhcpsettings{'FIX_ENABLED'},$dhcpsettings{'FIX_NEXTADDR'},$dhcpsettings{'FIX_FILENAME'},".
		"$dhcpsettings{'FIX_ROOTPATH'},$dhcpsettings{'FIX_REMARK'},$dhcpsettings{'FIX_HOSTNAME'},".
		"$dhcpsettings{'FIX_ROUTER'},$dhcpsettings{'FIX_DNS'}\n";
	    $dhcpsettings{'KEY2'} = '';       # End edit mode
	    &General::log($Lang::tr{'fixed ip lease modified'});
	}

        #Write changes to dhcpd.conf.
        &sortcurrent2;    # sort newly added/modified entry
        &buildconf;       # before calling buildconf which use fixed lease file !
    }
}

#
# Look for all cgi variables names matching "IP-macaddress" format 
# and create if necessary a fixed lease for it (the plus operator)
#
# This var will passed hidden to mark the new lease
my $newmacs = '';
if ($dhcpsettings{'ACTION_ALL'} eq $Lang::tr{'dhcp create fixed leases'}) {
    foreach (keys %dhcpsettings) {
        if (/^(\d+\.\d+\.\d+\.\d+)!([0-9a-fA-F:]+)!(.*)$/) {     # checked names are index of the line
            my $ip=$1;
            my $mac=$2;
	    my $comment = $3 || 'imported';

            if (!grep (/$2/,@current2) && General::validip($ip) ) {		    # new mac address

		my $proposed_ip = 0;
		#determine witch interface $ip belongs to so that we use the auto_start_lease or end_addr
		foreach my $itf (@ITFs) {
		    if ($dhcpsettings{"ENABLE_${itf}"} eq 'on' &&
		        &General::IpInSubnet ( $ip,
						$netsettings{"${itf}_NETADDRESS"},
						$netsettings{"${itf}_NETMASK"})) {

			    $proposed_ip  = $dhcpsettings{"BASE_IP_FIXED_LEASE_${itf}"} ?
					    $dhcpsettings{"BASE_IP_FIXED_LEASE_${itf}"} :
					    &General::NextIP( $dhcpsettings{"END_ADDR_${itf}"} );
		    }
		}

		if (! $proposed_ip) {	# use the current ip not 'ranged'
		    $proposed_ip = $ip;
		} else {		# starting a prposed_ip, increment it til not referenced in lease table
		    while ( grep (/$proposed_ip/, @current2)) {
			$proposed_ip = &General::NextIP( $proposed_ip );
		    }
		}
                unshift (@current2, "$mac,$proposed_ip,on,,,,$comment\n");
                $newmacs .= "$mac!";
            }
        }
    }
    if ($newmacs) {
        #Write changes to dhcpd.conf.
        $warnNTPmessage = $Lang::tr{'fixed ip lease added'} . ' (' . length($newmacs)/18 . ')';
        &General::log($warnNTPmessage);
        &sortcurrent2;    # sort newly added/modified entry
        &buildconf;       # before calling buildconf which use fixed lease file !
    }
}
if ($dhcpsettings{'ACTION'} eq $Lang::tr{'edit'}.'2') {
    #move out new line
    my $line = @current2[$dhcpsettings{'KEY2'}];
    chomp($line);
    ($dhcpsettings{'FIX_MAC'},
    $dhcpsettings{'FIX_ADDR'},
    $dhcpsettings{'FIX_ENABLED'},
    $dhcpsettings{'FIX_NEXTADDR'},
    $dhcpsettings{'FIX_FILENAME'},
    $dhcpsettings{'FIX_ROOTPATH'},
    $dhcpsettings{'FIX_REMARK'},
    $dhcpsettings{'FIX_HOSTNAME'},
    $dhcpsettings{'FIX_ROUTER'},
    $dhcpsettings{'FIX_DNS'})=split(',', $line);
}

if ($dhcpsettings{'ACTION'} eq $Lang::tr{'remove'}.'2') {
    splice (@current2,$dhcpsettings{'KEY2'},1);
    open(FILE, ">$filename2") or die 'Unable to open fixed lease file.';
    print FILE @current2;
    close(FILE);
    $dhcpsettings{'KEY2'} = '';				# End remove mode
    &General::log($Lang::tr{'fixed ip lease removed'});
    #Write changes to dhcpd.conf.
    &buildconf;
}
#end KEY2 defined

if ($dhcpsettings{'ACTION'} eq '' ) { # First launch from GUI

    # Set default DHCP values only if blank and disabled
    foreach my $itf (@ITFs) {
	if ($dhcpsettings{"ENABLE_${itf}"} ne 'on' ) {
	    $dhcpsettings{"DNS1_${itf}"} = $netsettings{"${itf}_ADDRESS"};
	    $dhcpsettings{"DEFAULT_LEASE_TIME_${itf}"} = '60';
	    $dhcpsettings{"MAX_LEASE_TIME_${itf}"} = '120';
	    $dhcpsettings{"DOMAIN_NAME_${itf}"} = $mainsettings{'DOMAINNAME'};
	}
    }
    $dhcpsettings{'FIX_ENABLED'} = 'on';
}

&Header::openpage($Lang::tr{'dhcp configuration'}, 1, '');
&Header::openbigbox('100%', 'left', '', $errormessage);

if ($errormessage) {
    &Header::openbox('100%', 'left', $Lang::tr{'error messages'});
    print "<font class='base'>$errormessage&nbsp;</font>\n";
    &Header::closebox();
}
if ($warnNTPmessage) {
    $warnNTPmessage = "<font color=${Header::colourred}><b>$Lang::tr{'capswarning'}</b></font>: $warnNTPmessage";
}

&Header::openbox('100%', 'left', 'DHCP');
print "<form method='post' action='$ENV{'SCRIPT_NAME'}'>";

foreach my $itf (@ITFs) {
    my %checked=();
    $checked{'ENABLE'}{'on'} = ( $dhcpsettings{"ENABLE_${itf}"} ne 'on') ? '' : "checked='checked'";
    $checked{'ENABLEBOOTP'}{'on'} = ( $dhcpsettings{"ENABLEBOOTP_${itf}"} ne 'on') ? '' : "checked='checked'";

    if ($netsettings{"${itf}_DEV"} ne '' ) { # Show only defined interface
	my $lc_itf=lc($itf);
print <<END
<table width='100%'>
<tr>
    <td width='25%' class='boldbase'><b><font color='${lc_itf}'>$Lang::tr{"$lc_itf interface"}</font></b></td>
    <td class='base'>$Lang::tr{'enabled'}
    <input type='checkbox' name='ENABLE_${itf}' $checked{'ENABLE'}{'on'} /></td>
    <td width='25%' class='base'>$Lang::tr{'ip address'}/$Lang::tr{'netmask'}:</td><td><b>$netsettings{"${itf}_ADDRESS"}/$netsettings{"${itf}_NETMASK"}</b></td>
</tr><tr>
    <td width='25%' class='base'>$Lang::tr{'start address'}&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td width='25%'><input type='text' name='START_ADDR_${itf}' value='$dhcpsettings{"START_ADDR_${itf}"}' /></td>
    <td width='25%' class='base'>$Lang::tr{'end address'}&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td width='25%'><input type='text' name='END_ADDR_${itf}' value='$dhcpsettings{"END_ADDR_${itf}"}' /></td>
</tr><tr>
    <td class='base'>$Lang::tr{'default lease time'}</td>
    <td><input type='text' name='DEFAULT_LEASE_TIME_${itf}' value='$dhcpsettings{"DEFAULT_LEASE_TIME_${itf}"}' size='5' /></td>
    <td class='base'>$Lang::tr{'max lease time'}</td>
    <td><input type='text' name='MAX_LEASE_TIME_${itf}' value='$dhcpsettings{"MAX_LEASE_TIME_${itf}"}' size='5' /></td>
</tr><tr>
    <td class='base'>$Lang::tr{'dhcp base ip fixed lease'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='BASE_IP_FIXED_LEASE_${itf}' value='$dhcpsettings{"BASE_IP_FIXED_LEASE_${itf}"}' /></td>
</tr><tr>
    <td class='base'>$Lang::tr{'domain name suffix'}&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='DOMAIN_NAME_${itf}' value='$dhcpsettings{"DOMAIN_NAME_${itf}"}' /></td>
    <td>$Lang::tr{'dhcp allow bootp'}:</td>
    <td><input type='checkbox' name='ENABLEBOOTP_${itf}' $checked{'ENABLEBOOTP'}{'on'} /></td>
</tr><tr>
    <td class='base'>$Lang::tr{'primary dns'}</td>
    <td><input type='text' name='DNS1_${itf}' value='$dhcpsettings{"DNS1_${itf}"}' /></td>
    <td class='base'>$Lang::tr{'secondary dns'}&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='DNS2_${itf}' value='$dhcpsettings{"DNS2_${itf}"}' /></td>
</tr><tr>
    <td class='base'>$Lang::tr{'primary ntp server'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='NTP1_${itf}' value='$dhcpsettings{"NTP1_${itf}"}' /></td>
    <td class='base'>$Lang::tr{'secondary ntp server'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='NTP2_${itf}' value='$dhcpsettings{"NTP2_${itf}"}' /></td>
</tr><tr>
    <td class='base'>$Lang::tr{'primary wins server address'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='WINS1_${itf}' value='$dhcpsettings{"WINS1_${itf}"}' /></td>
    <td class='base'>$Lang::tr{'secondary wins server address'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='WINS2_${itf}' value='$dhcpsettings{"WINS2_${itf}"}' /></td>
</tr>
</table>
<hr />
END
;
    }# Show only defined interface
}#foreach itf
print <<END
<table width='100%'>
<tr>
    <td class='base' width='25%'><img src='/blob.gif' align='top' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
    <td class='base' width='30%'>$warnNTPmessage</td>
    <td width='40%' align='center'><input type='submit' name='ACTION' value='$Lang::tr{'save'}' /></td>
    <td width='5%' align='right'>
        <a href='${General::adminmanualurl}/services.html#services_dhcp' target='_blank'>
        <img src='/images/web-support.png' alt='$Lang::tr{'online help en'}' title='$Lang::tr{'online help en'}' /></a></td>
</tr>
</table>
</form>
END
;

&Header::closebox();

if ($warnmessage) {
    $warnmessage = "<font color=${Header::colourred}><b>$Lang::tr{'capswarning'}</b></font>: $warnmessage";
}
$errormessage = &DhcpCheckSyntax;
if ($errormessage) {
    &Header::openbox('100%', 'left', $Lang::tr{'error messages'});
    print "<font class='base'>$errormessage&nbsp;</font>\n";
    &Header::closebox();
}

&Header::openbox('100%', 'left', $Lang::tr{'dhcp advopt list'});
# DHCP Advanced options settings
my %checked=();
$checked{'ADVOPT_ENABLED'}{'on'} = ($dhcpsettings{'ADVOPT_ENABLED'} ne 'on') ? '' : "checked='checked'";

print "<form method='post' action='$ENV{'SCRIPT_NAME'}'><table width='100%'>";
my $buttontext = $Lang::tr{'add'};
if ($dhcpsettings{'KEY1'} ne '') {
    $buttontext = $Lang::tr{'update'};
    print "<tr><td class='boldbase'><b>$Lang::tr{'dhcp advopt edit'}</b></td></tr>";
} else {
    print "<tr><td class='boldbase'><b>$Lang::tr{'dhcp advopt add'}</b></td></tr>"
}

#search if the 'option' is in the list and print the syntax model
my $opt = `/bin/grep "^option $dhcpsettings{'ADVOPT_NAME'} " $filename3`;
if ($opt ne '') {
   $opt =~ s/option $dhcpsettings{'ADVOPT_NAME'}/Syntax:/;  # "option xyz abc" => "syntax: abc"
   $opt =~ s/;//;
   #$opt = "<tr><td></td><td></td><td colspan='2'>$opt</td></tr>";
}
my $listopt = &SelectKnownOptions('ADVOPT_LIST');
print <<END
<tr>
    <td class='base'>$Lang::tr{'dhcp advopt name'}:</td>
    <td colspan='2'><input type='text' name='ADVOPT_NAME' value='$dhcpsettings{'ADVOPT_NAME'}' size='18' /> &nbsp;$Lang::tr{'or'} $Lang::tr{'select'} $listopt</td>
</tr><tr>
    <td class='base'>$Lang::tr{'dhcp advopt value'}:</td>
    <td><input type='text' name='ADVOPT_DATA' value='$dhcpsettings{'ADVOPT_DATA'}' size='60' />&nbsp;$opt</td>
</tr><tr>
    <td class='base'>$Lang::tr{'enabled'}<input type='checkbox' name='ADVOPT_ENABLED' $checked{'ADVOPT_ENABLED'}{'on'} /></td>
    <td class='base'>$Lang::tr{'dhcp advopt scope'}:&nbsp;<img src='/blob.gif' alt='*' />
    
END
;

# Put a checkbox for each interface. Checkbox visible disabled if interface is disabled  
foreach my $itf (@ITFs) {
    my $lc_itf=lc($itf);
    $checked{'ADVOPT_SCOPE_${itf}'}{'on'} = $dhcpsettings{"ADVOPT_SCOPE_${itf}"} ne 'on' ? '' : "checked='checked'";    
    print "$Lang::tr{\"${lc_itf}\"} <input type='checkbox' name='ADVOPT_SCOPE_${itf}' $checked{'ADVOPT_SCOPE_${itf}'}{'on'} ";
    print $dhcpsettings{"ENABLE_${itf}"} eq 'on' ? "/>" : "disabled='disabled' />";
    print "&nbsp; &nbsp;";
}

print <<END
    </td>
</tr>
</table>
<hr />
<table width='100%'>
<tr>
    <td class='base' width='35%'><img src='/blob.gif' align='top' alt='*' />&nbsp;$Lang::tr{'dhcp advopt scope help'}</td>
    <td class='base' width='25%'>$warnmessage</td>
    <td width='40%' align='center'>
    <input type='hidden' name='ACTION' value='$Lang::tr{'add'}1' />
    <input type='submit' name='SUBMIT' value='$buttontext' />
    <input type='submit' name='SUBMIT' value='$Lang::tr{'dhcp advopt help'}' />
    <input type='hidden' name='KEY1' value='$dhcpsettings{'KEY1'}' />
    </td>
</tr>
</table>
</form>
END
;
#Edited line number (KEY1) passed until cleared by 'save' or 'remove' or 'new sort order'

# print help taken from the file describing options
if ($dhcpsettings{'SUBMIT'} eq $Lang::tr{'dhcp advopt help'}) {
    print "<hr />";
    print "<table width='100%'>";
    print "<tr><td width='30%'><b>$Lang::tr{'dhcp advopt name'}</b></td><td width='70%'><b>$Lang::tr{'dhcp advopt value'}</b></td>";
    open(FILE, "$filename3");
    my @current3 = <FILE>;
    close(FILE);
    foreach my $line (@current3) {
	$line =~ /option ([a-z0-9-]+) (.*);/;
	print "<tr><td>$1</td><td>$2</td></tr>\n";
    }
    print "<tr><td colspan='2'><hr /></td></tr>\n";
    print '<tr><td>string type</td><td>"quoted string" or 00:01:FF...</td></tr>';
    print '<tr><td>text type</td><td>"quoted text only"</td></tr>';
    print '<tr><td>ip-address type </td><td>10.0.0.1 | www.dot.com</td></tr>';
    print '<tr><td>int,uint types</td><td>numbers</td></tr>';
    print '<tr><td>flag type</td><td>on | off</td></tr>';
    print '<tr><td>boolean type</td><td>on|true | off|false</td></tr>';
    print '</table>';
    print "<hr />";
    print "<table width='100%'>";
    print "<tr><td width='30%'><b>$Lang::tr{'dhcp advopt custom definition'}</b></td><td width='70%'><b>$Lang::tr{'dhcp advopt value'}</b></td>";
    print "<tr><td>any-name </td><td> code NNN=$OptionTypes</td></tr>";
    print '<tr><td>a-string</td><td>code 100=string</td></tr>';
    print '<tr><td>a-number</td><td>code 101=signed integer 8</td></tr>';
    print '<tr><td>an-array</td><td>code 102=array of "one basic type", except text&string</td></tr>';
    print '<tr><td>a-record</td><td>code 103={any,number,of,basics,types}</td></tr>';
    print '<tr><td>an-array-of-record</td><td>code 104=array of {any,number,of,basics,types}</td></tr>';
    print '<tr><td>wpad</td><td>code 252=text</td></tr>';
    print '<tr><td>wpad</td><td>"http://www.server.fr/path-to/proxy.pac"</td></tr>';
    print '<tr><td>myopt</td><td>code 152=array of {integer 16,boolean}</td></tr>';
    print '<tr><td>myopt</td><td>8 false, 0 true, 7 false</td></tr>';
    print '</table>';
}

print <<END
<hr />
<table width='100%'>
<tr>
    <td width='30%' class='boldbase' align='center'><b>$Lang::tr{'dhcp advopt name'}</b></td>
    <td width='50%' class='boldbase' align='center'><b>$Lang::tr{'dhcp advopt value'}</b></td>
    <td width='20%' class='boldbase' align='center'><b>$Lang::tr{'dhcp advopt scope'}</b></td>
    <td colspan='3' class='boldbase' align='center'><b>$Lang::tr{'action'}</b></td>
</tr>
END
;
my $key = 0;
foreach my $line (@current1) {
    my $gif = '';
    my $gdesc = '';
    chomp($line);   # remove newline
    my @temp = split(/\t/,$line);

    if ($temp[0] eq "on") {
	$gif = 'on.gif';
	$gdesc = $Lang::tr{'click to disable'};
    } else {
	$gif = 'off.gif';
	$gdesc = $Lang::tr{'click to enable'}; 
    }

    if ($dhcpsettings{'KEY1'} eq $key) {
	print "<tr bgcolor='${Header::colouryellow}'>";
    } elsif ($key % 2) {
	print "<tr bgcolor='${Header::table2colour}'>";
    } else {
	print "<tr bgcolor='${Header::table1colour}'>"; 
    }

    print <<END
<td align='center'>$temp[1]</td>
<td align='center'>$temp[2]</td>
<td align='center'>
END
;
    # Prepare a global flag to make easy reading
    my $global = '';
    my $disabledTogle = '';
    my $disabledEditRemove = '';
    if ( ExistNewOptionDefinition ($temp[1] . ' ' . $temp[2]) ) {
	$global = $Lang::tr{'dhcp advopt definition'};
	$disabledTogle = "disabled='disabled'";
	# Search if it is a used NewOptionDefinition to also disable edit & delete
	$disabledEditRemove = "disabled='disabled'" if (IsUsedNewOptionDefinition ($temp[1], $temp[2]));
    } else {
	$global = $Lang::tr{'dhcp advopt scope global'};
    }
    
    
    # Print each checked interface
    for (my $key=0; $key<@ITFs; $key++) {
	my $itf = $temp[3+$key];
	if ($itf ne 'off') {	# Only if an interface name is read
	    print "$itf";
	    $global='';		# fall to local scope !
	}
    }
    print <<END
$global</td>
<td align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ACTION' value='$Lang::tr{'toggle enable disable'}1' />
<input $disabledTogle type='image' name='$Lang::tr{'toggle enable disable'}' src='/images/$gif' alt='$gdesc' title='$gdesc' />
<input type='hidden' name='KEY1' value='$key' />
</form>
</td>

<td align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ACTION' value='$Lang::tr{'edit'}1' />
<input $disabledEditRemove type='image' name='$Lang::tr{'edit'}' src='/images/edit.gif' alt='$Lang::tr{'edit'}' title='$Lang::tr{'edit'}' />
<input type='hidden' name='KEY1' value='$key' />
</form>
</td>

<td align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ACTION' value='$Lang::tr{'remove'}1' />
<input $disabledEditRemove type='image' name='$Lang::tr{'remove'}' src='/images/delete.gif' alt='$Lang::tr{'remove'}' title='$Lang::tr{'remove'}' />
<input type='hidden' name='KEY1' value='$key' />
</form>
</td>
</tr>
END
;
    $key++;
}

print "</table>";

# If there are dhcp options, print Key to action icons
if ($key) {
print <<END
<table>
<tr>
	<td class='boldbase'>&nbsp;<b>$Lang::tr{'legend'}:&nbsp;</b></td>
	<td><img src='/images/on.gif' alt='$Lang::tr{'click to disable'}' /></td>
	<td class='base'>$Lang::tr{'click to disable'}</td>
	<td>&nbsp;&nbsp;</td>
	<td><img src='/images/off.gif' alt='$Lang::tr{'click to enable'}' /></td>
	<td class='base'>$Lang::tr{'click to enable'}</td>
	<td>&nbsp;&nbsp;</td>
	<td><img src='/images/edit.gif' alt='$Lang::tr{'edit'}' /></td>
	<td class='base'>$Lang::tr{'edit'}</td>
	<td>&nbsp;&nbsp;</td>
	<td><img src='/images/delete.gif' alt='$Lang::tr{'remove'}' /></td>
	<td class='base'>$Lang::tr{'remove'}</td>
</tr>
</table>
END
;
}
&Header::closebox();

&Header::openbox('100%', 'left', $Lang::tr{'current fixed leases'});
# Fixed leases screens
$checked{'FIX_ENABLED'}{'on'} = ($dhcpsettings{'FIX_ENABLED'} ne 'on') ? '' : "checked='checked'";
$dhcpsettings{'FIX_FILENAME'} =~ s/&comma;/,/g;
$dhcpsettings{'FIX_ROOTPATH'} =~ s/&comma;/,/g;

$buttontext = $Lang::tr{'add'};
print "<form method='post' action='$ENV{'SCRIPT_NAME'}'><table width='100%'>";

if ($dhcpsettings{'KEY2'} ne '') {
    $buttontext = $Lang::tr{'update'};
    print "<tr><td class='boldbase'><b>$Lang::tr{'edit an existing lease'}</b></td><td class='base'>$Lang::tr{'enabled'}<input type='checkbox' name='FIX_ENABLED' $checked{'FIX_ENABLED'}{'on'} /></td></tr>";
} else {
    print "<tr><td class='boldbase'><b>$Lang::tr{'add new lease'}</b></td><td class='base'>$Lang::tr{'enabled'}<input type='checkbox' name='FIX_ENABLED' $checked{'FIX_ENABLED'}{'on'} /></td></td></tr>"
}
print <<END
<tr>
    <td class='base'>$Lang::tr{'mac address'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_MAC' value='$dhcpsettings{'FIX_MAC'}' size='18' /></td>
    <td class='base'>$Lang::tr{'ip address'}:</td>
    <td><input type='text' name='FIX_ADDR' value='$dhcpsettings{'FIX_ADDR'}' size='18' /></td>
    <td class='base'>$Lang::tr{'remark'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_REMARK' value='$dhcpsettings{'FIX_REMARK'}' size='18' /></td>
</tr><tr>
    <td class='base'>$Lang::tr{'hostname'} $Lang::tr{'or'} FQDN:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_HOSTNAME' value='$dhcpsettings{'FIX_HOSTNAME'}' size='18' /></td>

    <td class='base'>$Lang::tr{'router ip'}&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_ROUTER' value='$dhcpsettings{'FIX_ROUTER'}' size='18' /></td>

    <td class='base'>$Lang::tr{'dns server'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_DNS' value='$dhcpsettings{'FIX_DNS'}' size='18' /></td>
END
;
print <<END
    
</tr><tr>
    <td colspan = '3'><b>$Lang::tr{'dhcp bootp pxe data'}</b></td>
</tr><tr>
    <td class='base'>next-server:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_NEXTADDR' value='$dhcpsettings{'FIX_NEXTADDR'}' size='18' /></td>
    <td class='base'>filename:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_FILENAME' value='$dhcpsettings{'FIX_FILENAME'}' size='18' /></td>
    <td class='base'>root-path:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td><input type='text' name='FIX_ROOTPATH' value='$dhcpsettings{'FIX_ROOTPATH'}' size='18' /></td>
</tr>
</table>
<hr />
<table width='100%'>
<tr>
    <td class='base' width='50%'><img src='/blob.gif' align='top' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}
    &nbsp;&nbsp;$Lang::tr{'dhcp fixed lease help1'}</td>
    <td width='50%' align='center'>
	<input type='hidden' name='ACTION' value='$Lang::tr{'add'}2' />
	<input type='submit' name='SUBMIT' value='$buttontext' />
	<input type='hidden' name='KEY2' value='$dhcpsettings{'KEY2'}' /></td>
</tr>
</table>
</form>
END
;
#Edited line number (KEY2) passed until cleared by 'save' or 'remove' or 'new sort order'

# Add visual indicators to column headings to show sort order - EO
my $sortarrow1 = '';
my $sortarrow2 = '';

if ($dhcpsettings{'SORT_FLEASELIST'} eq 'FETHERRev') {
	$sortarrow1 = $Header::sortdn;
} elsif ($dhcpsettings{'SORT_FLEASELIST'} eq 'FETHER') {
	$sortarrow1 = $Header::sortup;
} elsif ($dhcpsettings{'SORT_FLEASELIST'} eq 'FIPADDRRev') {
	$sortarrow2 = $Header::sortdn;
} else {
	$sortarrow2 = $Header::sortup;
}

print <<END
<hr />
<table width='100%'>
<tr>
    <td width='13%' align='center'><a href='$ENV{'SCRIPT_NAME'}?FETHER'><b>$Lang::tr{'mac address'}</b></a> $sortarrow1</td>
    <td width='13%' align='center'><a href='$ENV{'SCRIPT_NAME'}?FIPADDR'><b>$Lang::tr{'ip address'}</b></a> $sortarrow2</td>
    <td width='14%' class='boldbase' align='center'><b>$Lang::tr{'hostname'}</b></td>
    <td width='15%' align='center'><b>$Lang::tr{'remark'}</b></td>
    <td width='15%' class='boldbase' align='center'><b>next-server</b></td>
    <td width='15%' class='boldbase' align='center'><b>filename</b></td>
    <td width='15%' class='boldbase' align='center'><b>root-path</b></td>
    <td colspan='3' class='boldbase' align='center'><b>$Lang::tr{'action'}</b></td>
</tr>
END
;
my $ipdup = 0;
my %ipinuse = ();
my %macdupl = (); # Duplicate MACs have to be on different subnets
my %ipoutside = ();

# mark duplicate ip or duplicate MAC
foreach my $line (@current2) {
    my ($FIX_MAC, $FIX_ADDR) = split(',', $line);
    $macdupl{$FIX_MAC} += 1;
    if ($macdupl{$FIX_MAC} > 1) {
	$ipdup = 1; 	# Flag up duplicates for use later
    }
    $ipinuse{$FIX_ADDR} += 1;
    if ($ipinuse{$FIX_ADDR} > 1) {
	$ipdup = 1; 	# Flag up duplicates for use later
    }
    # Mark IP addresses outwith known subnets
    $ipoutside{$FIX_ADDR} = 1;
    foreach my $itf (@ITFs) {
        if ( &General::IpInSubnet($FIX_ADDR,
                $netsettings{"${itf}_NETADDRESS"}, 
                $netsettings{"${itf}_NETMASK"})) {
            $ipoutside{$FIX_ADDR} = 0;
        }
    }
}

$key = 0;
foreach my $line (@current2) {
    my $gif = '';
    my $gdesc = '';
    chomp($line);   # remove newline
    my ($FIX_MAC,
    $FIX_ADDR,
    $FIX_ENABLED,
    $FIX_NEXTADDR,
    $FIX_FILENAME,
    $FIX_ROOTPATH,
    $FIX_REMARK,
    $FIX_HOSTNAME,
    $FIX_ROUTER,
    $FIX_DNS) = split(',', $line);

    if ($FIX_ENABLED eq "on") {
	$gif = 'on.gif';
	$gdesc = $Lang::tr{'click to disable'};
    } else {
	$gif = 'off.gif';
	$gdesc = $Lang::tr{'click to enable'}; 
    }

    if ($dhcpsettings{'KEY2'} eq $key) {
	print "<tr bgcolor='${Header::colouryellow}'>";
    } elsif ($key % 2) {
	print "<tr bgcolor='${Header::table2colour}'>";
    } else {
	print "<tr bgcolor='${Header::table1colour}'>";
    }
    my $TAG0 = '';
    my $TAG1 = '';
    my $TAG2 = '';
    my $TAG3 = '';
    my $TAG4 = '';
    if ($ipinuse{$FIX_ADDR} > 1) {
	$TAG0 = '<b>';
	$TAG1 = '</b>';
    }
    if (($macdupl{$FIX_MAC} > 1) || ( grep (/$FIX_MAC/, $newmacs) ) ) {
	$TAG2 = '<b>';
	$TAG3 = '</b>';
    }
    if ($ipoutside{$FIX_ADDR} > 0) {
	$TAG4 = "bgcolor='orange'" if ($dhcpsettings{'KEY2'} ne $key);
    }
    # uncode  commas if field is not empty
    $FIX_FILENAME =~ s/&comma;/,/g;
    $FIX_ROOTPATH =~ s/&comma;/,/g;

    print <<END
<td align='center'>$TAG2$FIX_MAC$TAG3</td>
<td align='center' $TAG4>$TAG0$FIX_ADDR$TAG1</td>
<td align='center'>$FIX_HOSTNAME&nbsp;</td>
<td align='center'>$FIX_REMARK&nbsp;</td>
<td align='center'>$FIX_NEXTADDR&nbsp;</td>
<td align='center'>$FIX_FILENAME&nbsp;</td>
<td align='center'>$FIX_ROOTPATH&nbsp;</td>

<td align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ACTION' value='$Lang::tr{'toggle enable disable'}2' />
<input type='image' name='$Lang::tr{'toggle enable disable'}' src='/images/$gif' alt='$gdesc' title='$gdesc' />
<input type='hidden' name='KEY2' value='$key' />
</form>
</td>

<td align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ACTION' value='$Lang::tr{'edit'}2' />
<input type='image' name='$Lang::tr{'edit'}' src='/images/edit.gif' alt='$Lang::tr{'edit'}' title='$Lang::tr{'edit'}' />
<input type='hidden' name='KEY2' value='$key' />
</form>
</td>

<td align='center'>
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ACTION' value='$Lang::tr{'remove'}2' />
<input type='image' name='$Lang::tr{'remove'}' src='/images/delete.gif' alt='$Lang::tr{'remove'}' title='$Lang::tr{'remove'}' />
<input type='hidden' name='KEY2' value='$key' />
</form>
</td>
</tr>
END
;
    $key++;
}
print "</table>";

# If the fixed lease file contains entries, print Key to action icons
if ($key) {
my $dup = $ipdup ? "<td class='base'>$Lang::tr{'duplicate ip bold'}</td>" :'';
print <<END
<table>
<tr>
	<td class='boldbase'>&nbsp;<b>$Lang::tr{'legend'}:&nbsp;</b></td>
	<td><img src='/images/on.gif' alt='$Lang::tr{'click to disable'}' /></td>
	<td class='base'>$Lang::tr{'click to disable'}</td>
	<td>&nbsp;&nbsp;</td>
	<td><img src='/images/off.gif' alt='$Lang::tr{'click to enable'}' /></td>
	<td class='base'>$Lang::tr{'click to enable'}</td>
	<td>&nbsp;&nbsp;</td>
	<td><img src='/images/edit.gif' alt='$Lang::tr{'edit'}' /></td>
	<td class='base'>$Lang::tr{'edit'}</td>
	<td>&nbsp;&nbsp;</td>
	<td><img src='/images/delete.gif' alt='$Lang::tr{'remove'}' /></td>
	<td class='base'>$Lang::tr{'remove'}</td>
</tr>
<tr>
	<td>&nbsp;</td>
	<td bgcolor='orange'>&nbsp;</td>
	<td class='base'>$Lang::tr{'ip address outside subnets'}</td>
	<td>&nbsp;</td>
	<td>&nbsp;</td>
	$dup
</tr>
</table>
END
;
}

&Header::closebox();

foreach my $itf (@ITFs) {
    if ($dhcpsettings{"ENABLE_${itf}"} eq 'on') {
	# display leases with a list of actions to do with the global select checkbox.
	&Header::PrintActualLeases($Lang::tr{'dhcp create fixed leases'});
	last; 			                # Print one time only for all interfaces
    };
}

&Header::closebigbox();
&Header::closepage();

## Ouf it's the end !

sub sortcurrent1  # by now, do not sort, just write
{
    open(FILE, ">$filename1") or die 'Unable to open dhcp advanced options file.';
    print FILE @current1;
    close(FILE);
}


# Sort the "current2" array according to choices
sub sortcurrent2
{
    our %entries = ();

    sub fixedleasesort {
	my $qs='';
	if (rindex ($dhcpsettings{'SORT_FLEASELIST'},'Rev') != -1) {
	    $qs=substr ($dhcpsettings{'SORT_FLEASELIST'},0,length($dhcpsettings{'SORT_FLEASELIST'})-3);
	    if ($qs eq 'FIPADDR') {
		my @a = split(/\./,$entries{$a}->{$qs});
		my @b = split(/\./,$entries{$b}->{$qs});
		($b[0]<=>$a[0]) ||
		($b[1]<=>$a[1]) ||
		($b[2]<=>$a[2]) ||
		($b[3]<=>$a[3]);
	    } else {
		$entries{$b}->{$qs} cmp $entries{$a}->{$qs};
	    }
	} else { #not reverse
	    $qs=$dhcpsettings{'SORT_FLEASELIST'};
	    if ($qs eq 'FIPADDR') {
		my @a = split(/\./,$entries{$a}->{$qs});
		my @b = split(/\./,$entries{$b}->{$qs});
		($a[0]<=>$b[0]) ||
		($a[1]<=>$b[1]) ||
		($a[2]<=>$b[2]) ||
		($a[3]<=>$b[3]);
	    } else {
		$entries{$a}->{$qs} cmp $entries{$b}->{$qs};
	    }
	}
    }

    #Use an associative array (%entries)
    foreach my $line (@current2) {
	chomp( $line); #remove newline because can be on field 5 or 6 (addition of REMARK)
        my ($FIX_MAC, $FIX_ADDR, $DATA) = split(',', $line,3);
	my @record = ('FETHER',$FIX_MAC,'FIPADDR',$FIX_ADDR,'DATA',$DATA);
	my $record = {};                        # create a reference to empty hash
	%{$record} = @record;                	# populate that hash with @record
	# use combination of ether & IP as key to allow duplicates in either but not both
	$entries{$record->{FETHER} . $record->{FIPADDR}} = $record; # add this to a hash of hashes
    }

    open(FILE, ">$filename2") or die 'Unable to open fixed lease file.';
    foreach my $entry ( sort fixedleasesort keys %entries) {
	print FILE "$entries{$entry}->{FETHER},$entries{$entry}->{FIPADDR},$entries{$entry}->{DATA}\n";
    }
    close(FILE);

    # Reload sorted  @current2
    open (FILE, "$filename2");
    @current2 = <FILE>;
    close (FILE);
    undef (%entries);  #This array is reused latter. Clear it.
}

# Build the configuration file mixing  settings, fixed leases and advanced options/parameters
sub buildconf {
    open(FILE, ">/${General::swroot}/dhcp/dhcpd.conf") or die "Unable to write dhcpd.conf file";
    flock(FILE, 2);

    # Global settings
    print FILE "ddns-update-style none;\n";
    print FILE "deny bootp;	#default\n";
    print FILE "authoritative;\n";

    # Write first new option definition
    foreach my $line (@current1) {
	chomp($line);   # remove newline
	my @temp = split(/\t/,$line);
        if (ExistNewOptionDefinition ($temp[1] . ' ' . $temp[2])) {
		print FILE "option $temp[1] $temp[2];\n";
	}
    }
    # Write other global options
    foreach my $line (@current1) {
	chomp($line);   # remove newline
	my @temp = split(/\t/,$line);

	if ($temp[0] eq 'on' && !ExistNewOptionDefinition ($temp[1] . ' ' . $temp[2])){ # active & !definition
	    my $global=1;
	    for (my $key=0; $key<@ITFs; $key++) {
		my $itf = $temp[3+$key];
		if ($itf ne 'off') # Only if an interface name is read
		{
		    $global=0;
		}
	    }
	    if ($global) {
                #differentiate between options & parameters
                print FILE "option " if (! `/bin/grep "^$temp[1]\$" $filename4`);
		print FILE "$temp[1] $temp[2];\n";
	    }
	}# on    
    }# foreach line

    #Subnet range definition
    foreach my $itf (@ITFs) {
	my $lc_itf=lc($itf);
	if ($dhcpsettings{"ENABLE_${itf}"} eq 'on' ){
	    print FILE "\nsubnet " . $netsettings{"${itf}_NETADDRESS"} . " netmask ". $netsettings{"${itf}_NETMASK"} . " #$itf\n";
	    print FILE "{\n";
	    print FILE "\trange " . $dhcpsettings{"START_ADDR_${itf}"} . ' ' . $dhcpsettings{"END_ADDR_${itf}"}.";\n" if ($dhcpsettings{"START_ADDR_${itf}"});
	    print FILE "\toption subnet-mask "   . $netsettings{"${itf}_NETMASK"} . ";\n";
	    print FILE "\toption domain-name \"" . $dhcpsettings{"DOMAIN_NAME_${itf}"} . "\";\n";
	    print FILE "\toption routers " . $netsettings{"${itf}_ADDRESS"} . ";\n";
	    print FILE "\toption domain-name-servers " . $dhcpsettings{"DNS1_${itf}"}  if ($dhcpsettings{"DNS1_${itf}"});
	    print FILE ", " . $dhcpsettings{"DNS2_${itf}"}                             if ($dhcpsettings{"DNS2_${itf}"});
	    print FILE ";\n"                                                           if ($dhcpsettings{"DNS1_${itf}"});
	    print FILE "\toption ntp-servers " . $dhcpsettings{"NTP1_${itf}"}          if ($dhcpsettings{"NTP1_${itf}"});
	    print FILE ", " . $dhcpsettings{"NTP2_${itf}"}                             if ($dhcpsettings{"NTP2_${itf}"});
	    print FILE ";\n"                                                           if ($dhcpsettings{"NTP1_${itf}"});
	    print FILE "\toption netbios-name-servers " . $dhcpsettings{"WINS1_${itf}"}if ($dhcpsettings{"WINS1_${itf}"});
	    print FILE ", " . $dhcpsettings{"WINS2_${itf}"}                            if ($dhcpsettings{"WINS2_${itf}"});
	    print FILE ";\n"                                                           if ($dhcpsettings{"WINS1_${itf}"});
	    print FILE "\tdefault-lease-time " . ($dhcpsettings{"DEFAULT_LEASE_TIME_${itf}"} * 60). ";\n";
	    print FILE "\tmax-lease-time "     . ($dhcpsettings{"MAX_LEASE_TIME_${itf}"} * 60)    . ";\n";
	    print FILE "\tallow bootp;\n" if ($dhcpsettings{"ENABLEBOOTP_${itf}"} eq 'on');

	    # Write scoped options
	    foreach my $line (@current1) {
		chomp($line);   # remove newline
		my @temp = split(/\t/,$line);		# Use TAB separator !

		if ($temp[0] eq 'on'){
		    for (my $key=0; $key<@ITFs; $key++) {
			if ($itf eq $temp[3+$key]) # Only if an interface name is read
			{
                            #differentiate between options & parameters
			    print FILE "\t";
                            print FILE "option " if (! `/bin/grep "^$temp[1]\$" $filename4`);
		            print FILE "$temp[1] $temp[2];\n";
			}
		    }
		}# on
	    }# foreach line
	    print FILE "} #$itf\n";

	    system ('/bin/touch', "${General::swroot}/dhcp/enable_${lc_itf}");
	    &General::log("DHCP on ${itf}: " . $Lang::tr{'dhcp server enabled'})
	} else {
	    unlink "${General::swroot}/dhcp/enable_${lc_itf}";
	    &General::log("DHCP on ${itf}: " . $Lang::tr{'dhcp server disabled'})
	}
    }

    #write fixed leases if any. Does not handle duplicates to write them elsewhere than the global scope.
    my $key = 0;
    foreach my $line (@current2) {
	chomp($line);
        my ($FIX_MAC,
        $FIX_ADDR,
        $FIX_ENABLED,
        $FIX_NEXTADDR,
        $FIX_FILENAME,
        $FIX_ROOTPATH,
        $FIX_REMARK,
        $FIX_HOSTNAME,
        $FIX_ROUTER,
        $FIX_DNS) = split(',', $line);

	if ($FIX_ENABLED eq "on") {
	    print FILE "\nhost " . ($FIX_HOSTNAME ? $FIX_HOSTNAME: "fix$key") . " # $FIX_REMARK\n";
	    print FILE "{\n";
	    print FILE "\thardware ethernet $FIX_MAC;\n"    if ($FIX_MAC);
	    print FILE "\tfixed-address $FIX_ADDR;\n";
	    print FILE "\tnext-server $FIX_NEXTADDR;\n"     if ($FIX_NEXTADDR);
	    $FIX_FILENAME =~ s/&comma;/,/g;
	    print FILE "\tfilename \"$FIX_FILENAME\";\n"    if ($FIX_FILENAME);
	    $FIX_ROOTPATH =~ s/&comma;/,/g;
	    print FILE "\toption root-path \"$FIX_ROOTPATH\";\n" if ($FIX_ROOTPATH);
	    if ($FIX_HOSTNAME) {
		my ($opt_host, $opt_dom) = split (/\./, $FIX_HOSTNAME, 2);
		print FILE "\toption host-name \"$opt_host\";\n" ;
		print FILE "\toption domain-name \"$opt_dom\";\n" if ($opt_dom);
	    }
	    print FILE "\toption routers $FIX_ROUTER;\n" if ($FIX_ROUTER);
	    print FILE "\toption domain-name-servers $FIX_DNS;\n" if ($FIX_DNS);
	    print FILE "}\n";
	    $key++;
	}
    }
    close FILE;
    system '/usr/local/bin/restartdhcp';
}

#
# Receive a string and if it matches model for a new option,
# add it to the list %NewOptions
#
my %NewOptions = ();

sub AddNewOptionDefinition {
    my ($opt) = @_;
    if ( $opt =~ /^([-\w]+)( code \d+=($OptionTypes))/ ) {
	$NewOptions{$1} = $2;
	#&General::log ("new:<$1><$2>");
	return 1;
    }
    return 0;
}

#
# Check existence of definition for a new option
#
sub ExistNewOptionDefinition {
    my ($opt) = @_;
    if ( $opt =~ /^([-\w]+)( code \d+=($OptionTypes))/ ) {
	return defined $NewOptions{$1};
    }
    return 0;
}

#
# Check if it is a new option (definition must exist)
# "code \d+=" test eliminates a false response when definition exists
# but this string is the definition.
sub ValidNewOption {
    my ($opt) = @_;
    if ($opt =~ /^([-\w]+) (.*)/ ) {
	return defined ( $NewOptions{$1} ) && $2 !~ /code \d+=/;
    }
    return 0;
}

#
# Check if the new option $opt is used, except the definition of itself!
#
sub IsUsedNewOptionDefinition {
    my ($opt) = @_;
    foreach my $line (@current1) {
	#chomp($line);   # remove newline        #don't know why, but this remove newline in @current1 .... !
	my @temp = split(/\t/,$line);
	# if we find something "opt value" & value != "code nnn=" it's ok.
	return 1 if ( ($opt eq $temp[1]) && ($temp[2] !~ /code \d+=/) );
    }
    return 0;
}

#
# Build a Select with option names listed in $filename3 & OptionDefinition
#
sub SelectKnownOptions {
    # Create the SELECT with valid values from filename3
    my $select_varname = shift;
    my $select = "<select name='$select_varname'>";
    $select .= "<option value=''>---</option>";

    $select .= "<optgroup label='Parameters'>";
    open(FILE, "$filename4");
    my @lines = <FILE>;
    close(FILE);
    foreach my $line (@lines) {
        $select .= "<option value='$line'>$line</option>";
    }

    $select .= "<optgroup label='Options'>";
    open(FILE, "$filename3");
    @lines = <FILE>;
    close(FILE);
    foreach my $line (@lines) {
	my @name=split(' ',$line);
        $select .= "<option value='$name[1]'>$name[1]</option>";
    }

    # Add to the Select OptionDefinition
    $select .= "<optgroup label='Options(yours)'>";
    foreach my $line (keys %NewOptions) {
        $select .= "<option value='$line'>$line</option>";
    }
    $select .= "</select>\n";
    return ($select);
}

#
# Launch a test to verify the syntax of the dhcpd.conf
# Return empty or the fault message
sub DhcpCheckSyntax {
    my @ret = `/usr/sbin/dhcpd -t 2>&1`;
    my $i;
    for ($i=0; $i<@ret; $i++) {
        #look for the error condition
        if ( $ret[$i] =~ /^Configuration file errors encountered/ )
        {
            return "<pre>$ret[$i-3]$ret[$i-2]$ret[$i-1]</pre>";
        }
    }
    return '';
}


sub myvalidiporfqdn {
    my $test = shift;
    my $prev = shift || '';
    my $err  = '';
    $err = $Lang::tr{'invalid fixed ip address'}.":&nbsp;$test"
    	if (! &General::validiporfqdn($test));

    if ($err && $prev) { $err = "$prev<br>$err"; }
    elsif ($prev) {$err=$prev};
    return $err;
}
