#!/usr/bin/perl
#
# SmoothWall CGIs
#
# This code is distributed under the terms of the GPL
#
# (c) The SmoothWall Team
#
# $Id: ids.cgi,v 1.8.2.40 2008/06/16 08:30:22 eoberlander Exp $
#

use strict;

# enable only the following on debugging purpose
#use warnings; no warnings 'once';# 'redefine', 'uninitialized';
#use CGI::Carp 'fatalsToBrowser';

require 'CONFIG_ROOT/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";

my %snortsettings=();
my %snorttags=();	# separate from settings to not been include in backup/restore
my %checked=();
my %netsettings=();
our $errormessage = '';
my $logfile = '/var/log/snort/lastrulesupdate';
my $rulestargz = '/var/log/snort/rules.tar.gz';

# to fill some parts of the <table>
my $dl_disabled='';
my $apply_message='';
my $load_date='';
my $md5_date='';

&General::readhash("${General::swroot}/ethernet/settings", \%netsettings);

$snortsettings{'ENABLE_SNORT'} = 'off'; #this is historically for RED
$snortsettings{'ENABLE_SNORT_GREEN'} = 'off';
$snortsettings{'ENABLE_SNORT_BLUE'} = 'off';
$snortsettings{'ENABLE_SNORT_ORANGE'} = 'off';
$snortsettings{'ACTION'} = '';
$snortsettings{'RULESTYPE'} = 'nothing';
$snortsettings{'OINKCODE'} = '';
&Header::getcgihash(\%snortsettings);

# Installed rules
# only one tag for (subscripted|registered) as we update only one at a time
# if we update both tags, we will not know if the other is more recent or older
$snorttags{'VRTMERGEDATE'} = '';	# last time oinkmaster merge rules
 # reading md5 of rules file is allowed only once every 15 mn so we need to track when was the last time to avoid error
$snorttags{'MD5PROBEVALUE'} = '';	# file md5 value
$snorttags{'VRTLOADDATE'} = 0;		# last VRT file load
$snorttags{'VRTLOADMD5'} = '';		# last loaded VRT file md5
$snorttags{'MD5PROBEDATE'} = 0;		# last md5 file read
$snorttags{'APPLY'} = '';
$snorttags{'MESSAGE'} = '';
&General::readhash('/etc/snort/rulestags', \%snorttags);

if ($snortsettings{'ACTION'} eq $Lang::tr{'save'}) {
	&General::writehash("${General::swroot}/snort/settings", \%snortsettings);
	if ($snortsettings{'ENABLE_SNORT'} eq 'on') {
		system ('/bin/touch', "${General::swroot}/snort/enable");
	} else {
		unlink "${General::swroot}/snort/enable";
	} 
	if ($snortsettings{'ENABLE_SNORT_GREEN'} eq 'on') {
		system ('/bin/touch', "${General::swroot}/snort/enable_green");
	} else {
		unlink "${General::swroot}/snort/enable_green";
	} 
	if ($snortsettings{'ENABLE_SNORT_BLUE'} eq 'on') {
		system ('/bin/touch', "${General::swroot}/snort/enable_blue");
	} else {
		unlink "${General::swroot}/snort/enable_blue";
	} 
	if ($snortsettings{'ENABLE_SNORT_ORANGE'} eq 'on') {
		system ('/bin/touch', "${General::swroot}/snort/enable_orange");
	} else {
		unlink "${General::swroot}/snort/enable_orange";
	}
	# keep track that settings has been changed until 'apply' is used
	$snorttags{'APPLY'} = 'NEEDED';
	&General::writehash('/etc/snort/rulestags', \%snorttags);
}
# read settings after 'save' could have written new settings (and possibly unselect checkboxes)
&General::readhash("${General::swroot}/snort/settings", \%snortsettings);

# form is empty on first display so check after settings has been read
if ( $snortsettings{'RULESTYPE'} ne 'nothing' && 
   ( length($snortsettings{'OINKCODE'})!=40 || $snortsettings{'OINKCODE'} !~ /^[a-z0-9]+$/ ) ) {
	$errormessage = $Lang::tr{'invalid input for oink code'};
}
if ($snortsettings{'ACTION'} eq $Lang::tr{'download new ruleset'}) {
	system('/usr/local/bin/snortrules','--load');
	# keep track that settings has been changed until 'apply' is used
	$snorttags{'APPLY'} = 'NEEDED';
} elsif ($snortsettings{'ACTION'} eq $Lang::tr{'refresh update list'}) {
	system('/usr/local/bin/snortrules','--md5');
} elsif ($snortsettings{'ACTION'} eq $Lang::tr{'apply'}) {
	&General::readhash('/etc/snort/rulestags', \%snorttags);
	$snorttags{'APPLY'} = '';
	$snorttags{'MESSAGE'}='';
	my $merge_error=0;
	if (-s $rulestargz) {
		# a new ruleset is available
		if (!system('/usr/local/bin/snortrules','--merge')) {
			# write there the tag
			my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
			$snorttags{'VRTMERGEDATE'} = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $mday, $hour, $min , $sec);
		} else {
			# should be snortrules run more than once at the same time
			# hide the log this time as it is not complete before previous action finish
			$snortsettings{'ACTION'} = '';
		}
	}
	my $fail=(system('/usr/local/bin/restartsnort','red','orange','blue','green')>>8);
	if ($fail) { $errormessage="$fail $Lang::tr{'snort failure(s) to start'}"; }
	&General::writehash('/etc/snort/rulestags', \%snorttags);
}
# read tags after they may have been updated by snortrules.pl (load or md5) or erased by apply action
&General::readhash('/etc/snort/rulestags', \%snorttags);

# priority to errormessage from web interface, then from snortrules.pl
if ($errormessage eq '') { $errormessage=$snorttags{'MESSAGE'}; }

if ($snorttags{'APPLY'} eq 'NEEDED' ) { $apply_message=$Lang::tr{'apply warning'}; }

# prepare tags display for the right place in the table
if ($snortsettings{'RULESTYPE'} =~ /^(registered|subscripted)$/) {
	if ($snorttags{'VRTLOADDATE'}!=0) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($snorttags{'VRTLOADDATE'});
		$load_date = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $mday, $hour, $min , $sec);
	}
	if ($snorttags{'MD5PROBEDATE'}!=0) {
		my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($snorttags{'MD5PROBEDATE'});
		$md5_date= sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year+1900, $mon+1, $mday, $hour, $min , $sec);
	}
}

# enabled download depending of the settings
unless ($snortsettings{'RULESTYPE'} =~ /^(registered|subscripted)$/ &&
		length($snortsettings{'OINKCODE'})==40 && $snortsettings{'OINKCODE'} =~ /^[a-z0-9]+$/ ) {
	$dl_disabled="disabled='disabled'";
}

# usage warning on low memory
my ($swapsize, $swapused, $swapfree);
my ($ram,$size,$used,$free,$shared,$buffers,$cached);
open(FREE,'/usr/bin/free -o |');
while(<FREE>) {
	if ($_ =~ m/^Mem:\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/) {
		($ram,$size,$used,$free,$shared,$buffers,$cached) = ($1,$1,$2,$3,$4,$5,$6);
	}
	if ($_ =~ m/^Swap:\s+(\d+)\s+(\d+)\s+(\d+)$/) { ($swapsize,$swapused,$swapfree) = ($1,$2,$3); }
}
close FREE;

# display running snort and memory used
my $iface = '';
my $red_status = '';
if (open(FILE, "${General::swroot}/red/iface")) {
	$iface = <FILE>;
	close FILE;
	chomp $iface;
	$red_status=&General::isrunning("snort_$iface");
}
my $green_status=&General::isrunning("snort_$netsettings{'GREEN_DEV'}");

$checked{'ENABLE_SNORT'}{'off'} = '';
$checked{'ENABLE_SNORT'}{'on'} = '';
$checked{'ENABLE_SNORT'}{$snortsettings{'ENABLE_SNORT'}} = "checked='checked'";
$checked{'ENABLE_SNORT_GREEN'}{'off'} = '';
$checked{'ENABLE_SNORT_GREEN'}{'on'} = '';
$checked{'ENABLE_SNORT_GREEN'}{$snortsettings{'ENABLE_SNORT_GREEN'}} = "checked='checked'";
$checked{'ENABLE_SNORT_BLUE'}{'off'} = '';
$checked{'ENABLE_SNORT_BLUE'}{'on'} = '';
$checked{'ENABLE_SNORT_BLUE'}{$snortsettings{'ENABLE_SNORT_BLUE'}} = "checked='checked'";
$checked{'ENABLE_SNORT_ORANGE'}{'off'} = '';
$checked{'ENABLE_SNORT_ORANGE'}{'on'} = '';
$checked{'ENABLE_SNORT_ORANGE'}{$snortsettings{'ENABLE_SNORT_ORANGE'}} = "checked='checked'";
$checked{'RULESTYPE'}{'nothing'} = '';
$checked{'RULESTYPE'}{'registered'} = '';
$checked{'RULESTYPE'}{'subscripted'} = '';
$checked{'RULESTYPE'}{$snortsettings{'RULESTYPE'}} = "checked='checked'";

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'intrusion detection system'}, 1, '');
&Header::openbigbox('100%', 'left', '', $errormessage);

if ($errormessage) {
	&Header::openbox('100%', 'left', $Lang::tr{'error messages'});
	print "<class name='base'>$errormessage\n";
	print "&nbsp;</class>\n";
	&Header::closebox();
}

if ($swapused>1000 && $free<10_000) {
	# values are arbitrary
	&Header::openbox('100%', 'left', $Lang::tr{'capswarning'});
	print "<class name='base'>$Lang::tr{'swap warning'} <a href='/cgi-bin/status.cgi'>$Lang::tr{'sssystem status'}</a>\n";
	print "&nbsp;</class>\n";
	&Header::closebox();
}

&Header::openbox('100%', 'left', "$Lang::tr{'intrusion detection system'}:");
print <<END
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<table border='0' width='100%'>
<tr>
	<th align='left'>$Lang::tr{'interfaces'}</th>
	<th align='center'>$Lang::tr{'status'}:</th>
	<th align='center'>$Lang::tr{'memory'}</th>
</tr>
<tr>
	<td class='base'><input type='checkbox' name='ENABLE_SNORT_GREEN' $checked{'ENABLE_SNORT_GREEN'}{'on'} />
		GREEN Snort $netsettings{'GREEN_DEV'}</td>
	$green_status
</tr>
END
;
if ($netsettings{'BLUE_DEV'} ne '') {
	my $blue_status=&General::isrunning("snort_$netsettings{'BLUE_DEV'}");
	print <<END
<tr>
	<td class='base'><input type='checkbox' name='ENABLE_SNORT_BLUE' $checked{'ENABLE_SNORT_BLUE'}{'on'} />
		BLUE Snort $netsettings{'BLUE_DEV'}</td>
	$blue_status
</tr>
END
;
}
if ($netsettings{'ORANGE_DEV'} ne '') {
	my $orange_status=&General::isrunning("snort_$netsettings{'ORANGE_DEV'}");
	print <<END
<tr>
	<td class='base'><input type='checkbox' name='ENABLE_SNORT_ORANGE' $checked{'ENABLE_SNORT_ORANGE'}{'on'} />
		ORANGE Snort $netsettings{'ORANGE_DEV'}</td>
	$orange_status
</tr>
END
;
}
print <<END
<tr>
	<td class='base'><input type='checkbox' name='ENABLE_SNORT' $checked{'ENABLE_SNORT'}{'on'} />
		RED Snort $iface</td>
	$red_status
</tr>
<tr>
	<td colspan='3'><hr /></td>
</tr>
<tr>
	<td colspan='3'>$Lang::tr{'ids rules license'} <a href='http://www.snort.org/' target='_blank'>http://www.snort.org</a>.<br />
		$Lang::tr{'ids rules license2'} <a href='http://www.snort.org/reg-bin/userprefs.cgi' target='_blank'>USER PREFERENCES</a>,
		$Lang::tr{'ids rules license3'}</td>
</tr>
<tr>
	<td nowrap='nowrap' colspan='3'>Oink Code:
		<input type='text' size='45' name='OINKCODE' value='$snortsettings{'OINKCODE'}' />
	</td>
</tr>
<tr>
	<th colspan='3' align='left'>$Lang::tr{'ids rules update'}:</th>
</tr>
<tr>
	<td colspan='3'><input type='radio' name='RULESTYPE' value='nothing' $checked{'RULESTYPE'}{'nothing'} />$Lang::tr{'no'}</td>
</tr>
<tr>
	<td nowrap='nowrap'><input type='radio' name='RULESTYPE' value='registered' $checked{'RULESTYPE'}{'registered'} />
		$Lang::tr{'registered user rules'}&nbsp;<img src='/blob.gif' alt='*' /></td>
	<td colspan='2' rowspan='2'><img src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'only one download every 15 mn'}</td>
</tr>
<tr>
	<td nowrap='nowrap'><input type='radio' name='RULESTYPE' value='subscripted' $checked{'RULESTYPE'}{'subscripted'} />
		$Lang::tr{'subscripted user rules'}</td>
</tr>
<tr>
	<td colspan='3'><hr /></td>
</tr>
<tr>
	<td colspan='3'><input type='submit' name='ACTION' value="$Lang::tr{'save'}" />&nbsp;$apply_message</td>
</tr>
<tr>
	<td nowrap='nowrap' colspan='3'><input type='submit' name='ACTION' value="$Lang::tr{'refresh update list'}" $dl_disabled />
		&nbsp;$md5_date</td>
</tr>
<tr>
	<td colspan='3'><input type='submit' name='ACTION' value="$Lang::tr{'download new ruleset'}" $dl_disabled />
		&nbsp;$load_date</td>
</tr>
<tr>
	<td colspan='3'><input type='submit' name='ACTION' value="$Lang::tr{'apply'}" />&nbsp;$snorttags{'VRTMERGEDATE'}</td>
</tr>
<tr>
	<td colspan='2'><input type='submit' name='ACTION' value="$Lang::tr{'read last rules installation log'}" $dl_disabled /></td>
	<td align='right'><a href='${General::adminmanualurl}/services.html#services_ids' target='_blank'>
		<img src='/images/web-support.png' alt='$Lang::tr{'online help en'}' title='$Lang::tr{'online help en'}' /></a>
	</td>
</tr>
</table>
</form>
END
;

# display update log
if ($snortsettings{'ACTION'} eq $Lang::tr{'read last rules installation log'} || $snortsettings{'ACTION'} eq $Lang::tr{'apply'} ) {
	print "<b>$Lang::tr{'installed updates'}</b>\n<pre>";
	open(FILE,$logfile) or die "Unable to open $logfile";
	my @log=<FILE>;
	close FILE;
	foreach my $line (@log) { print &Header::cleanhtml($line); }
	print "</pre>";
}

&Header::closebox();
&Header::closebigbox();
&Header::closepage();
