#!/usr/bin/perl
#
# SmoothWall CGIs
#
# This code is distributed under the terms of the GPL
#
# (c) The SmoothWall Team
#
# $Id: updates.cgi,v 1.9.2.37 2008/03/16 15:49:26 owes Exp $
#

use LWP::UserAgent;
use File::Copy;
use strict;

# enable only the following on debugging purpose
#use warnings; no warnings 'once';# 'redefine', 'uninitialized';
#use CGI::Carp 'fatalsToBrowser';

require 'CONFIG_ROOT/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";
use POSIX qw(uname);

# early define sub with prototypes;
sub download($);

my $errormessage='';
my @av=('');	#list of available patch
my @pf=('');	#list of installed patch
my @pending=('');# information from just uploaded patch
my @signature=('');# signature from just uploaded patch

sub get_error()
{
	my $exit_code = shift || return '';
	if ($exit_code == 0) { return '';
	} elsif($exit_code == 2) { return "$Lang::tr{'could not create directory'}";
	} elsif($exit_code == 3) { return "$Lang::tr{'this is not an authorised update'}";
	} elsif($exit_code == 4) { return "$Lang::tr{'this is not a valid archive'}";
	} elsif($exit_code == 5) { return "$Lang::tr{'could not open update information file'}";
	} elsif($exit_code == 6) { return "$Lang::tr{'could not open installed updates file'}";
	} elsif($exit_code == 7) { return "$Lang::tr{'this update is already installed'}";
	} elsif($exit_code == 11) { return "$Lang::tr{'not enough disk space'}";
	} else { return "$Lang::tr{'package failed to install'}";
	}
}

&Header::showhttpheaders();

my %uploadsettings=();
my $kernelrelease=(POSIX::uname())[2];
my $kernelused=$Lang::tr{'monoprocessor'};
$kernelused=$Lang::tr{'smp'} if ($kernelrelease =~ /smp/);

$uploadsettings{'KERNEL'} = '';
$uploadsettings{'ACTION'} = '';
&Header::getcgihash(\%uploadsettings, {'wantfile' => 1, 'filevar' => 'FH'});

# to display a friendly name
sub get_kernel_text()
{
	my $kernelselected=$Lang::tr{'no set selected'};
	if ($uploadsettings{'KERNEL'} eq 'MONO') {
		$kernelselected=$Lang::tr{'monoprocessor'};
	} elsif ($uploadsettings{'KERNEL'} eq 'SMP') {
		$kernelselected=$Lang::tr{'smp'};
	} elsif ($uploadsettings{'KERNEL'} eq 'BOTH') {
		$kernelselected=$Lang::tr{'both kernels'};
	}
	return $kernelselected;
}

if ($uploadsettings{'ACTION'} eq $Lang::tr{'save'}) {
	my $kernelselected=&get_kernel_text();
	# do not write to settings or that would break upload
	map (delete ($uploadsettings{$_}) ,'FH');
	if ($uploadsettings{'KERNEL'} ne 'BOTH') {
		if ($kernelselected ne $kernelused) {
			$errormessage="$Lang::tr{'running'} $kernelused <> $kernelselected";
		} else {
			&General::writehash("${General::swroot}/main/kernelsettings", \%uploadsettings);
			$errormessage = &get_error(system('/usr/local/bin/installpackage --strip >/dev/null')>>8);
			if (!$errormessage) {
				&General::log("$kernelselected $Lang::tr{'selected'}");
			}
		}
	}
}
# read settings after 'save' could have written new settings
# ${General::swroot}/patches directory is not used as exclude from backup
&General::readhash("${General::swroot}/main/kernelsettings", \%uploadsettings);

if ($uploadsettings{'ACTION'} eq $Lang::tr{'upload'}) {
	if (copy ($uploadsettings{'FH'}, '/var/patches/patch.tgz.gpg') != 1) {
		$errormessage = $!;
	} else {
		$errormessage = &get_error(system('/usr/local/bin/installpackage --read >/dev/null')>>8);
	}
} elsif ($uploadsettings{'ACTION'} eq $Lang::tr{'download'}) {
	# we do not use URL include in information for 2 reasons
	#- does not support different arch
	#- available list is not refreshed after update, so position of next update vary in the array
	# URL will remain for compatibility (at least on 1.4)
	my @tmp=split(/[.]/, $General::version);
	++$tmp[2];
	my $arch = (POSIX::uname())[4];
	if ($arch =~ /^alpha/) { $arch='alpha'; } else { $arch='i386'; }
	my $URL="http://prdownloads.sourceforge.net/ipcop/ipcop-$tmp[0].$tmp[1].$tmp[2]-update.$arch.tgz.gpg?download";
	my $databuf = &download($URL);
	if ($databuf && $databuf->is_success) {
		# write the datastream to a file
		open(FILE, '>/var/patches/patch.tgz.gpg');
		binmode(FILE);
		syswrite(FILE, $databuf->content);
		close(FILE);
		$errormessage = &get_error(system('/usr/local/bin/installpackage --read >/dev/null')>>8);
	} else {
		unless(-e "${General::swroot}/red/active") {
			$errormessage =$Lang::tr{'connection is down'};
		} else {
			$errormessage = "$! $URL";
		}
	}
} elsif ($uploadsettings{'ACTION'} eq $Lang::tr{'apply'}) {
	my $errormessage = &get_error(system('/usr/local/bin/installpackage --install >/dev/null')>>8);
	if ($errormessage eq '') {
		#Hack to get correct version displayed after update
		open (XX,"/usr/bin/perl -e \"require'${General::swroot}/general-functions.pl';print \\\$General::version\"|");
		$General::version=<XX>;
		close (XX);
	}
} elsif ($uploadsettings{'ACTION'} eq $Lang::tr{'delete'}) {
	my $errormessage = &get_error(system('/usr/local/bin/installpackage --delete >/dev/null')>>8);
} elsif ($uploadsettings{'ACTION'} eq $Lang::tr{'refresh update list'}) {
	my $return = &download("http://www.ipcop.org/patches/${General::version}");
	if ($return && $return->is_success) {
		if (open(LIST, ">${General::swroot}/patches/available")) {
			flock LIST, 2;
			my @this = split(/----START LIST----\n/,$return->content);
			print LIST $this[1];
			close(LIST);
			&General::log($Lang::tr{'successfully refreshed updates list'});
		} else {
			$errormessage = $Lang::tr{'could not open available updates file'};
		}
	} else {
		unless(-e "${General::swroot}/red/active") {
			$errormessage =$Lang::tr{'connection is down'};
		} else {
			$errormessage = $Lang::tr{'could not download the available updates list'};
		}
	}
} elsif ($uploadsettings{'ACTION'} eq "$Lang::tr{'clear cache'} (squid)") {
	system('/usr/local/bin/restartsquid','-f');
} elsif ($uploadsettings{'ACTION'} eq "$Lang::tr{'kernel selection'}") {
	if ($uploadsettings{'KERNEL'} eq '') {
		$uploadsettings{'KERNEL'}='MONO' if ($kernelused eq $Lang::tr{'monoprocessor'});
		$uploadsettings{'KERNEL'}='SMP' if ($kernelused eq $Lang::tr{'smp'});
	}
	# for option box
	my %selected=();
	$selected{'KERNEL'}{'MONO'} = '';
	$selected{'KERNEL'}{'SMP'} = '';
	$selected{'KERNEL'}{'BOTH'} = '';
	$selected{'KERNEL'}{$uploadsettings{'KERNEL'}} = "selected='selected'";
	&Header::openpage($Lang::tr{'kernel selection'}, 1, '');
	&Header::openbigbox('100%', 'left');
	&Header::openbox('100%', 'left', $Lang::tr{'kernel selection'});
	print <<EOF
<form method='post' action='$ENV{'SCRIPT_NAME'}'>
<table border='0'>
<tr>
	<td>$Lang::tr{'kernel explanation'}<br /></td>
</tr><tr>
	<td>$Lang::tr{'kernel message'} :$kernelused, linux-$kernelrelease</td>
</tr><tr>
	<td>$Lang::tr{'what should remain'}
	<select name='KERNEL'>
	<option value='MONO' $selected{'KERNEL'}{'MONO'}>$Lang::tr{'monoprocessor'}</option>
	<option value='SMP' $selected{'KERNEL'}{'SMP'}>$Lang::tr{'smp'}</option>
	<option value='BOTH' $selected{'KERNEL'}{'BOTH'}>$Lang::tr{'both kernels'}</option>
	</select></td>
</tr><tr>
	<td align='center'><input type='submit' name='ACTION' value='$Lang::tr{'save'}' />&nbsp;
		<input type='submit' name='ACTION' value='$Lang::tr{'cancel'}' /></td>
</tr>
</tr><tr>
	<td>$Lang::tr{'reboot to select boot option again'}</td>
</tr>
</table>
</form>
EOF
;
	&Header::closebox();
	&Header::closebigbox();
	&Header::closepage();
	exit (0);
}

# Read-in the list files installed/ready to install
if (!open(AV, "<${General::swroot}/patches/available")) {
	$errormessage = $Lang::tr{'could not open available updates file'};
} else {
	@av = <AV>;
	close(AV);
}
if (!open (PF, "<${General::swroot}/patches/installed")) {
	$errormessage = $Lang::tr{'could not open installed updates file'};
} else {
	@pf = <PF>;
	close (PF);
	#substract installed patch from list displayed
	# 'available' file has not changed on update installation
	foreach my $P (@pf) {
		$P =~ /^(...)/;
		my $order=$1;
		my $idx=0;
		foreach my $A (@av) {
			$A =~ /^(...)/;
			if ($1 eq $order) { # match
				splice (@av,$idx,1);
				last;
			}
			$idx++;
		}
	}
}
if (open(PENDING, '</var/patches/information')) {
	@pending= <PENDING>;
	close(PENDING);
}
if (open(SIGNATURE, '</var/patches/signature')) {
	@signature= <SIGNATURE>;
	close(PENDING);
}
&Header::openpage($Lang::tr{'updates'}, 1, '');
&Header::openbigbox('100%', 'left', 'download.png', $errormessage);

if ($errormessage) {
	&Header::openbox('100%', 'left', $Lang::tr{'error messages'});
	print $errormessage;
	print "&nbsp;";
	&Header::closebox();
}

&Header::openbox('100%', 'left', $Lang::tr{'available updates'});
print "<form method='post' action='/cgi-bin/updates.cgi' enctype='multipart/form-data'>";
if ( defined $av[0] ) {
	print $Lang::tr{'there are updates available'};
	print<<END
<table width='100%' border='0' cellpadding='2' cellspacing='0'>
    <tr>
	<td width='5%'><b>$Lang::tr{'id'}</b></td>
	<td width='10%'><b>$Lang::tr{'title'}</b></td>
	<td><b>$Lang::tr{'description'}</b></td>
	<td width='10%'><b>$Lang::tr{'released'}</b></td>
	<td width='10%'><b>$Lang::tr{'download'}</b></td>
    </tr>
END
;
	my $number=0;# display download button only on first update
	foreach (@av) {
		my @temp = split(/\|/,$_);
		$number++;
		#field4 is an URL, make a link.
		$temp[4] =~/^http.*\/(.*)\?/;	# get filename
		$temp[4] = "<a href='$temp[4]' target='_new'>$1</a>";
		if ($number==1) {
			$temp[4] .="<br ><br ><input type='submit' name='ACTION' value='$Lang::tr{'download'}' />";
		}
		print "<tr><td valign='top'>" . join("</td><td valign='top'>", @temp) . "</td></tr>";
	}
	print '</table>';
} else {
	print $Lang::tr{'all updates installed'};
}

print<<END
<hr />
<br />$Lang::tr{'to install an update'}<br />
<table>
    <tr>
	<td align='right' class='base'><b>$Lang::tr{'upload update file'}</b></td>
	<td><input type="file" size='40' name="FH" /><input type='submit' name='ACTION' value='$Lang::tr{'upload'}' /></td>
    </tr>
</table>
END
;
# check files exist and are uid root (stat(file))[4]=0 except .gpg file wich protected by signature
if (-e '/var/patches/information' && -e '/var/patches/patch.tgz.gpg' && -e '/var/patches/signature' &&
		(stat('/var/patches/information'))[4]+(stat('/var/patches/signature'))[4]==0) {
	$pending[0] =~ s/\|/<\/td><td valign='top'>/g;
	$signature[0] = &Header::cleanhtml($signature[0]);
	$signature[1] = &Header::cleanhtml($signature[1]);
	print <<END
<table width='100%' border='0'>
<tr>
	<td width='5%'><b>$Lang::tr{'id'}</b></td>
	<td width='10%'><b>$Lang::tr{'title'}</b></td>
	<td><b>$Lang::tr{'description'}</b></td>
	<td width='10%'><b>$Lang::tr{'released'}</b></td>
</tr><tr>
	<td valign='top'>$pending[0]<br /><br />
		<input type='submit' name='ACTION' value='$Lang::tr{'apply'}' /><br /><br />
		<input type='submit' name='ACTION' value='$Lang::tr{'delete'}' /></td>
</tr><tr>
	<td colspan='2'></td>
	<td><hr /></td>
</tr><tr>
	<td colspan='2'>&nbsp;</td>
	<td colspan='2'>$signature[0]<br />$signature[1]</td>
</tr>
</table>
END
	;
}

# To show free space on hard disk
open (XX,'/bin/df -B M -x rootfs|');
my @df=<XX>;
close (XX);
# skip first line:
# Filesystem            Size  Used Avail Use% Mounted on
shift(@df);
chomp(@df);
# discount possible patch.tgz.gpg size, (stat())[7] is size
my $patchsize=0;
$patchsize=(stat('/var/patches/patch.tgz.gpg'))[7]/1024/1024 if (-e '/var/patches/patch.tgz.gpg');
 # alert on rootfs available < 20MB, on boot 1MB, on /var/log 1MB
my @alert=(20-$patchsize, 1, 1);
my $kernelselected=&get_kernel_text();
print<<END
<hr />
<table border='0'>
<tr>
	<td colspan='7'><b>$Lang::tr{'disk usage'}</b></td>
	<td><b>$Lang::tr{'kernel selection'}:</b></td>
	<td>linux-$kernelrelease</td>
</tr><tr>
	<td align='left' class='boldbase'><b>$Lang::tr{'device'}</b></td>
	<td align='left' class='boldbase'><b>$Lang::tr{'mounted on'}</b></td>
	<td align='center' class='boldbase'><b>$Lang::tr{'size'}</b></td>
	<td align='center' class='boldbase'><b>$Lang::tr{'used'}</b></td>
	<td align='center' class='boldbase'><b>$Lang::tr{'free'}</b></td>
	<td align='left' class='boldbase'><b>$Lang::tr{'percentage'}</b></td>
	<td>&nbsp;</td>
	<td><input type='submit' name='ACTION' value="$Lang::tr{'kernel selection'}" /></td>
	<td>$kernelselected</td>
</tr>
END
;
my @kernelmsg=("$Lang::tr{'kernel advice rootfs'}",
		"$Lang::tr{'kernel advice boot partition'}",
		"<input type='submit' name='ACTION' value='$Lang::tr{'clear cache'} (squid)' />");
for (my $count = 0; $count<=$#df; $count++) {
	my $tmp=((@df))[$count];
	$tmp =~ /([\w\/]+)\s+(\d+)M\s+(\d+)M\s+(\d+)M\s+(\d+)%\s+([\w\/]+)/;
	my ($dev,$mount,$size,$used,$free,$percent)=($1,$6,$2,$3,$4,$5);
	my $alertstyle="";
	$alertstyle="style='background-color: #FF0000;'" if ($4 <= $alert[$count]);
print<<END
<tr>
	<td>$dev</td>
	<td>$mount</td>
	<td align='right'>$size</td>
	<td align='right'>$used</td>
	<td align='right' $alertstyle>$free</td>
	<td align='right'>$percent%</td>
	<td>&nbsp;</td>
	<td colspan='2'>$kernelmsg[$count]</td>
</tr>
END
	;
}
print<<END
</table>
<hr />
<table width='100%'>
<tr>
    <td class='base' valign='top'><!--<img src='/blob.gif' alt='*' />--></td>
    <td width='55%' class='base'><!--$Lang::tr{'this field may be blank'}--></td>
    <td width='40%' align='center'>
	<input type='submit' name='ACTION' value='$Lang::tr{'refresh update list'}' />    </td>
    <td width='5%' align='right'>
    <a href='${General::adminmanualurl}/system.html#updates' target='_blank'>
    <img src='/images/web-support.png' alt='$Lang::tr{'online help en'}' title='$Lang::tr{'online help en'}' /></a></td>
</tr>
</table>
</form>
END
;
&Header::closebox();

&Header::openbox('100%', 'left', $Lang::tr{'installed updates'});
print<<END
<table width='100%' border='0' cellpadding='2' cellspacing='0'>
    <tr>
    <td width='5%'><b>$Lang::tr{'id'}</b></td>
    <td width='10%'><b>$Lang::tr{'title'}</b></td>
    <td><b>$Lang::tr{'description'}</b></td>
    <td width='10%'><b>$Lang::tr{'released'}</b></td>
    <td width='10%'><b>$Lang::tr{'installed'}</b></td>
    </tr>
END
;

foreach my $pf (reverse(@pf)) {
	next if $pf =~ m/^#/;
	my @temp = split(/\|/,$pf);
	# if field4 is a date ok else replace the unknown string
	$temp[4] = '????' if ($temp[4] !~/^\d/);
	print "<tr><td valign='top'>" . join("</td><td valign='top'>", @temp) . "</td></tr>";
}
print "</table>";

&Header::closebox();
&Header::closebigbox();
&Header::closepage();

sub download($)
{
	unless (-e "${General::swroot}/red/active") {
		return 0;
	}
	my $URL = shift || die 'Missing URL';
	my $downloader = LWP::UserAgent->new;
	$downloader->timeout(30);

	my %proxysettings=();
	&General::readhash("${General::swroot}/proxy/settings", \%proxysettings);

	if ($_=$proxysettings{'UPSTREAM_PROXY'}) {
		my ($peer, $peerport) = (/^(?:[a-zA-Z ]+\:\/\/)?(?:[A-Za-z0-9\_\.\-]*?(?:\:[A-Za-z0-9\_\.\-]*?)?\@)?([a-zA-Z0-9\.\_\-]*?)(?:\:([0-9]{1,5}))?(?:\/.*?)?$/);
		if ($proxysettings{'UPSTREAM_USER'}) {
			$downloader->proxy("http","http://$proxysettings{'UPSTREAM_USER'}:$proxysettings{'UPSTREAM_PASSWORD'}@"."$peer:$peerport/");
		} else {
			$downloader->proxy("http","http://$peer:$peerport/");
		}
	}
	return $downloader->get($URL, 'Cache-Control', 'no-cache');
}
