/* IPCop install2 program.
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Lawrence Manning, 2001
 * (c) Franck Bourdonnec, 2006
 * Contains update/restore code
 * 
 * $Id: install2.c,v 1.1.2.20 2008/03/04 16:16:27 gespinasse Exp $
 * 
 */
#include "install.h"

FILE *flog = NULL;
char *mylog;
char **ctr;

#define RESTORESKIP 1
#define RESTOREFLOPPY 2
#define RESTOREUSB 3
#define RESTOREURL 4

/* dir to find files, chrooted or not... */
#define TMP_EXTRACT_CH "/tmp/ipcop"
#define TMP_EXTRACT "/harddisk" TMP_EXTRACT_CH
#define MOUNT_BACKUP_CH "/mnt/usb"
#define MOUNT_BACKUP "/harddisk" MOUNT_BACKUP_CH
#define DATFILE_CH "/home/httpd/html/backup/ipcop-xxxx-xx-xx_xx-xx-xx.dat"
#define DATFILE "/harddisk" DATFILE_CH

/* net.c */
int checktarball(char *, char *message);

/* form elements shared with callback */
newtComponent radioskip, radiofloppy, radiousb, radiourl;
newtComponent restoreform, entryhostname, entrypassword;

/* Return 0 when dev contains a backup key.
 * Leave dev mounted */
int try_mount (char *dev, char *hostname)
{
	char commandstring[STRING_SIZE];
	mysystem("/bin/umount " MOUNT_BACKUP " 2>/dev/null");
	sprintf(commandstring, "/bin/mount -t vfat -o ro %s " MOUNT_BACKUP, dev);
	mysystem(commandstring);

	/*verify it's what we want */
	sprintf(commandstring, MOUNT_BACKUP "/backup.%s.key", hostname);
	FILE *handle = fopen(commandstring, "r");
	if (handle == NULL) {
		return 1;   /* not backup on this dev */
	}
	fclose(handle);
	return 0; //success
}

/* try to mount usb device until backup.<hostname>.key is found except the 
    destination device (scsi names are identical with usb key)
    check "sda sdb sdc sdd"
*/
int mountbackup (char *hostname, char *destination_device)
{
	char sourcedev[30];
	char i,j;
	for (i = 'a'; i < 'e'; i++) {
		sprintf (sourcedev,"/dev/sd%c ",i);
		if (strcmp (destination_device, sourcedev) != 0) {
			if (!try_mount (sourcedev, hostname)) return 0;
		}
		for (j = '1'; j < '5'; j++) {
			sourcedev[8] = j;
			if (strcmp (destination_device, sourcedev) != 0) {
				if (!try_mount (sourcedev, hostname)) return 0;
			}
		}
	}
	return 1;
}

/* Check the SQUID acl file exists, if not use our 1.4 copy */
void fixup_squidacl()
{
	FILE *aclreadfile;
	if ((aclreadfile = fopen ("/harddisk" CONFIG_ROOT "/proxy/acl", "r"))) {
		unlink ("/harddisk" CONFIG_ROOT "/proxy/acl-1.4");
		fclose(aclreadfile);
	} else {
		rename ("/harddisk" CONFIG_ROOT "/proxy/acl-1.4",
			"/harddisk" CONFIG_ROOT "/proxy/acl");
	}
	chown  ("/harddisk" CONFIG_ROOT "/proxy/acl", 0, 0);
}
/* if we detected SCSI then fixup */
void fixup_initrd()
{
	FILE *handle;
	char line[STRING_SIZE];
	char commandstring[STRING_SIZE];
	
	if (!(handle = fopen("/scsidriver", "r"))) 
		return;

	char *driver;
	fgets(line, STRING_SIZE-1, handle);
	fclose(handle);
	line[strlen(line) - 1] = 0;
	driver = strtok(line, ".");
	fprintf(flog, "Detected SCSI driver %s\n", driver);
	if (!strlen(driver) > 1) 
		return;
	
	fprintf(flog, "Fixing up ipcoprd.img\n");
	mysystem("/bin/chroot /harddisk /sbin/modprobe loop");
	mkdir("/harddisk/initrd", S_IRWXU|S_IRWXG|S_IRWXO);
	sprintf(commandstring, "/bin/chroot /harddisk /sbin/mkinitrd"
				" --with=scsi_mod --with=%s --with=sd_mod"
				" --with=sr_mod --with=libata"
				" --with=ataraid /boot/ipcoprd.img "KERNEL_VERSION,
				driver );
	runcommandwithstatus(commandstring, ctr[TR_BUILDING_INITRD]);
#ifdef __i386__
	sprintf(commandstring, "/bin/chroot /harddisk /sbin/mkinitrd"
				" --with=scsi_mod --with=%s --with=sd_mod"
				" --with=sr_mod --with=libata"
				" --with=ataraid /boot/ipcoprd-smp.img "KERNEL_VERSION"-smp",
				driver );
	runcommandwithstatus(commandstring, ctr[TR_BUILDING_INITRD]);
	mysystem("/bin/chroot /harddisk /bin/mv /boot/grub/scsigrub.conf /boot/grub/grub.conf");
#endif
#ifdef __alpha__
	runcommandwithstatus("/bin/chroot /harddisk /bin/mv /boot/etc/scsiaboot.conf /boot/etc/aboot.conf", ctr[TR_BUILDING_INITRD]);
#endif
}
/* When backup is open in tmpdir, if it is really a backup,
   move files to definitive location else return 2
*/
int check_copy_files(void)
{

	FILE *handle;
	/* Verify it is a backup by reading var/ipcop/main/settings file */
	handle = fopen ("/harddisk" TMP_EXTRACT_CH "/var/" SNAME
			"/main/settings", "r");
	if (handle == NULL) {
		return 2; /* it could not be a backup, no settings file*/
	}
	fclose (handle);

	mysystem("/bin/chroot /harddisk /bin/cp -af "TMP_EXTRACT_CH"/. /");
	/* Upgrade necessary files from v1.2 to v1.3 to v1.4 */
	upgrade_v12_v13();
	upgrade_v130_v140();
	/* Upgrade configuration files starting from 1.4.11 */
	mysystem("/bin/chroot /harddisk /usr/local/bin/upgrade");
	return 0;
}

/* Return an integer that define restore type */
int getrestoretype(void)
{
	newtComponent selected = newtRadioGetCurrent(radioskip);

	return selected == radioskip ? RESTORESKIP : \
			selected == radiofloppy ? RESTOREFLOPPY : \
			selected == radiousb ? RESTOREUSB : \
			selected == radiourl ? RESTOREURL : 0;
}

void restoredialogcallbacktype(newtComponent cm, void *data)
{
	int restoretype = getrestoretype();

	if (restoretype==RESTOREUSB || restoretype==RESTOREURL) {
		newtEntrySetFlags(entryhostname, NEWT_FLAG_DISABLED, NEWT_FLAGS_RESET);
		newtEntrySetFlags(entrypassword, NEWT_FLAG_DISABLED, NEWT_FLAGS_RESET);
	} else {
		newtEntrySetFlags(entryhostname, NEWT_FLAG_DISABLED, NEWT_FLAGS_SET);
		newtEntrySetFlags(entrypassword, NEWT_FLAG_DISABLED, NEWT_FLAGS_SET);

	}
	newtRefresh();
	newtDrawForm(restoreform);
}

int main(int argc, char *argv[]) {
#define LANG argv[1]
#define DEST_DEV argv[2]
#define WGET argv[3]

#ifdef	LANG_EN_ONLY
	char **langtrs[] = { en_tr, NULL };
#else
	char **langtrs[] = { af_tr, bz_tr, cs_tr, da_tr, de_tr, en_tr, es_tr, fr_tr, el_tr, it_tr, la_tr, hu_tr, nl_tr, no_tr, pl_tr, pt_tr, sk_tr, so_tr, fi_tr, sv_tr, tr_tr, vi_tr, NULL };
#endif

	char message[1000];
	char commandstring[STRING_SIZE];
	char *hostnamevalue;
	char *passwordvalue;
	char hostnameinit[STRING_SIZE] = SNAME ".localdomain";
	FILE *handle;

	/* create the GUI screen and objects */
	newtComponent header, labelhostname, labelpassword, ok;

	struct newtExitStruct es;
	int restoretype = 0;

	setlocale (LC_ALL, "");
	/* Log file/terminal stuff. */
	mylog = "/dev/tty2";
	ctr = langtrs[ atoi(LANG) ];

	if (!(flog = fopen(mylog, "w+"))) {
		printf("Couldn't open log terminal\n");
		return 0;
	}
	fprintf(flog, "Install2 program started.\n");
	newtInit();
	newtCls();
	newtDrawRootText(14, 0, get_title());
	newtPushHelpLine(ctr[TR_HELPLINE]);

	/* working dirs... */
	mkdir(MOUNT_BACKUP, S_IRWXU|S_IRWXG|S_IRWXO);

	restoreform = newtForm (NULL, NULL,0);

	sprintf(message, ctr[TR_RESTORE_CONFIGURATION], NAME);
	header = newtTextboxReflowed (2,1,message,51,0,0,0);
	newtFormAddComponent(restoreform, header);

	radioskip = newtRadiobutton (17, 5, ctr[TR_SKIP], 0, NULL);
	radiofloppy = newtRadiobutton (17, 6, ctr[TR_FLOPPY], 1, radioskip);
	radiousb = newtRadiobutton (17, 7, ctr[TR_USB_KEY], 0, radiofloppy);

	newtComponentAddCallback(radioskip,restoredialogcallbacktype, NULL);
	newtComponentAddCallback(radiofloppy,restoredialogcallbacktype, NULL);
	newtComponentAddCallback(radiousb,restoredialogcallbacktype, NULL);

	if (strcmp(WGET,"none")) {
		radiourl = newtRadiobutton (17, 8, "http/ftp", 0, radiousb);
		newtFormAddComponents(restoreform, radioskip, radiofloppy, radiousb, radiourl, NULL);
		newtComponentAddCallback(radiourl,restoredialogcallbacktype, NULL);
	} else {
		/* Network is not yet set, so url can't be used */
		newtFormAddComponents(restoreform, radioskip, radiofloppy, radiousb, NULL);
		radiourl = NULL;
	}

	/* hostname to search for backup key and backup .dat */
	labelhostname=newtTextbox(12, 10, 35, 1, 0);
	newtTextboxSetText (labelhostname, ctr[TR_HOSTNAME]);
	newtFormAddComponent(restoreform, labelhostname);
	entryhostname = newtEntry (13, 11, hostnameinit, 35, &hostnamevalue, 0);
	newtFormAddComponent(restoreform, entryhostname);

	/* password for the key */
	labelpassword=newtTextbox(12, 13, 35, 1, 0);
	newtTextboxSetText (labelpassword, ctr[TR_BACKUP_PASSWORD]);
	newtFormAddComponent(restoreform, labelpassword);
	entrypassword = newtEntry (13, 14, "", 20, &passwordvalue, 0);
	/* draw '*'  of chars in entrybox */
	newtEntrySetFlags(entrypassword, NEWT_FLAG_PASSWORD, NEWT_FLAGS_SET);
	newtFormAddComponent(restoreform, entrypassword);

	/* not used during default floppy selection */
	newtEntrySetFlags(entryhostname, NEWT_FLAG_DISABLED, NEWT_FLAGS_SET);
	newtEntrySetFlags(entrypassword, NEWT_FLAG_DISABLED, NEWT_FLAGS_SET);

	/* OK button */
	ok=newtButton(23, 16, ctr[TR_OK]);
	newtFormAddComponent(restoreform, ok);

	/* loop until it succeed (retcode=0) or user skip out (retcode=1) */
	int retcode = -1;
	do {

	DISPLAYFORM:
		newtCenteredWindow (55,20,ctr[TR_RESTORE]);
		newtFormRun(restoreform, &es);
		newtPopWindow();

		if (es.u.co != ok) {
			goto DISPLAYFORM; /* do we have other choice ?.... */
		}
		/* OK was pressed; verify the contents of each entry. */
		restoretype=getrestoretype();
		strcpy(hostnameinit,hostnamevalue);

		if (restoretype==RESTORESKIP) {
			retcode = 1;	// return 'no restore made' to install
			break;          // out of the do while loop
		}
		

		mkdir(TMP_EXTRACT, S_IRWXU|S_IRWXG|S_IRWXO);
		statuswindow(45, 4, get_title(), ctr[TR_READING_BACKUP]);
		switch (restoretype) {
		case RESTOREURL:
			sprintf(commandstring, "/bin/wget -O " TMP_EXTRACT "/backup.key"
				" %s/backup.%s.key", WGET, hostnamevalue);
			if (mysystem (commandstring)) {
				newtPopWindow();
				sprintf(commandstring, "%s/backup.%s.key", WGET, hostnamevalue);
				sprintf(message, ctr[TR_FILE_NOT_FOUND], commandstring);
				errorbox(message);
				break;  //end  of the case
			}
			/* use fixed name for .dat until we are sure it's the right hostname */
			sprintf(commandstring,
				"/bin/wget -O " DATFILE " %s/%s.dat",WGET, hostnamevalue);
			if (mysystem (commandstring)) {
				newtPopWindow();
				sprintf(commandstring, "%s/%s.dat", WGET, hostnamevalue);
				sprintf(message, ctr[TR_FILE_NOT_FOUND], commandstring);
				errorbox(message);
				break;  //end  of the case
			}
			goto COMMON;

		case RESTOREUSB:
			if (mountbackup( hostnamevalue, DEST_DEV )) {
				newtPopWindow();
				errorbox(ctr[TR_NO_BACKUP_ON_USB_FOUND]);
				break; //end  of the case
			};
			/* link files to a COMMON location if found */
			sprintf (commandstring,
				MOUNT_BACKUP "/backup.%s.key", hostnamevalue);
			if (!(handle = fopen(commandstring, "r"))) {
				newtPopWindow();
				sprintf(commandstring, "backup.%s.key", hostnamevalue);
				sprintf(message, ctr[TR_FILE_NOT_FOUND], commandstring);
				errorbox(message);
				break; 
			}
			fclose(handle);
			sprintf (commandstring,
				"chroot /harddisk ln -s "
					MOUNT_BACKUP_CH "/backup.%s.key "
					TMP_EXTRACT_CH "/backup.key", hostnamevalue);
			mysystem (commandstring);

			sprintf (commandstring, MOUNT_BACKUP "/%s.dat", hostnamevalue);
			if (!(handle = fopen(commandstring, "r"))) {
				newtPopWindow();
				sprintf(commandstring, "%s.dat", hostnamevalue);
				sprintf(message, ctr[TR_FILE_NOT_FOUND], commandstring);
				errorbox(message);
			}
			fclose(handle);
			/* use a fixed name for .dat until we are sure it's the right hostname */
			sprintf (commandstring,
				"chroot /harddisk cp -f "
					MOUNT_BACKUP_CH "/%s.dat "
					DATFILE_CH, hostnamevalue);
			mysystem (commandstring);

		COMMON:
			/* Decrypt the key on usual location */
			if (!strcmp(passwordvalue, "")) {
				/* password is mandatory to decrypt the key */
				newtPopWindow();
				errorbox(ctr[TR_PASSWORD_CANNOT_BE_BLANK]);
				break; //end  of the case
			}
			sprintf(commandstring,  "/bin/chroot /harddisk /usr/bin/openssl enc"
						" -a -d -aes256 -salt"
						" -pass pass:%s"
						" -in " TMP_EXTRACT_CH "/backup.key"
						" -out " CONFIG_ROOT "/backup/backup.key",
						passwordvalue);
			if (mysystem(commandstring)) {
				newtPopWindow();
				errorbox(ctr[TR_WRONG_PASSWORD_OR_KEYFILE]);
				break;//end  of the case
			}
			mysystem("/bin/chroot /harddisk /bin/chmod 400 "
				CONFIG_ROOT "/backup/backup.key");

			sprintf(commandstring, "/bin/chroot /harddisk"
				" /usr/local/bin/ipcoprscfg --restore " DATFILE_CH " --hardware");
			retcode = mysystem(commandstring);
			if (retcode) {
				newtPopWindow();
				errorbox(ctr[TR_UNABLE_TO_INSTALL_FILES]);//invalid backup
				break; //end  of the case
			}
			/* remove crypted version */
			unlink(TMP_EXTRACT "/backup.key");
			unlink(DATFILE);
			/* in any case we close the main windows. */
			newtPopWindow();
			break; //end  of the case
		case RESTOREFLOPPY:
			/* Run updfstab to locate a floppy.
			   This should be run each time the user tries to restore
			   so it can properly detect removable devices */
			if (mysystem("/bin/chroot /harddisk /usr/sbin/updfstab")) {
				newtPopWindow();
				errorbox(ctr[TR_UNABLE_TO_WRITE_ETC_FSTAB]);
				break; //end  of the case
			}

			/* Always extract to /tmp/ipcop for temporary extraction
			   just in case floppy fails.
			   Try a compressed backup first because it's quicker to fail.
			   In exclude.system, file names must be without leading / or 
			   on extraction, name will never match
			*/
			sprintf(commandstring, "/bin/chroot /harddisk /bin/tar -X "
						CONFIG_ROOT "/backup/exclude.system"
						" -C " TMP_EXTRACT_CH
						" -xvzf /dev/floppy");
			if (mysystem(commandstring)) {
				/* if it's not compressed, try uncompressed first before failing*/
				sprintf(commandstring, "/bin/chroot /harddisk /bin/tar -X "
							CONFIG_ROOT "/backup/exclude.system"
							"-C " TMP_EXTRACT_CH
							" -xvf /dev/floppy");
				if (mysystem(commandstring)) {
					/* command failed trying to read from floppy */
					newtPopWindow();
					errorbox(ctr[TR_UNABLE_TO_INSTALL_FILES]);
					break; //end  of the case
				}
			}
			/* If it is a backup, copy to correct location
			   else give another chance */
			retcode = check_copy_files();
			/* in any case we close the main windows. */
			newtPopWindow();
			if (retcode) 
				errorbox(ctr[TR_NO_MAIN_SETTINGS_IN_BACKUP]);
		}//switch
		/* remove possible badly restored files */
		mysystem("/bin/chroot /harddisk /bin/rm -rf " TMP_EXTRACT_CH );
	} while (retcode != 0);
	newtFormDestroy(restoreform);

	/* cleanup */
	mysystem("/bin/umount " MOUNT_BACKUP);
	mysystem("/bin/chroot /harddisk /bin/rmdir " MOUNT_BACKUP_CH);

	/* others operations moved from install to install2 */
	fixup_squidacl();
	fixup_initrd();

	fprintf(flog, "Install2 program ended.\n");
	fflush(flog);
	fclose(flog);
	newtFinished();
	return retcode;
}
