#!/bin/sh

# When a module loads ok, write module name and exit immediatly.
try_and_leave() {
    echo "Trying $1"; 
    if /bin/modprobe $1 > /dev/null 2>&1; then
	echo "$1.o.gz" > /scsidriver
	#/bin/cat /proc/scsi/scsi
	cd $OLDDIR
	exit 0
    fi
}

# Try each scsi module and don't return when one is found
try_all_modules() {

    # spread in drivers...
    for i in mptscsih cpqarray cciss DAC960 medley hptraid pdcraid silraid
    do
	try_and_leave $i
    done

    # the raid drivers autloaded ataraid. Remove it (might conflicts...)
    /bin/rmmod ataraid > /dev/null 2>&1

    #in kernel scsi dir...
    for i in * ;
    do
	# Skip the generic scsi modules and ancillary support modules
	# Added eata_dma to skip list because it crashes some machines. Probe last.
	if [ $i != "scsi_mod.o.gz" -a $i != "sd_mod.o.gz" \
	    -a $i != "sg.o.gz" -a $i != "sr_mod.o.gz" -a $i != "53c700.o.gz" \
	    -a $i != "NCR53C9x.o.gz" -a $i != "eata_dma.o.gz" -a $i != "libata.o.gz" ]; then
	    DRIVER=`echo $i | sed 's/.o.gz//'`
	    try_and_leave $DRIVER
	fi
    done

    # sometimes crashes the machine, put last
    try_and_leave eata_dma
}

load_common() {
    /bin/modprobe scsi_mod.o.gz > /dev/null 2>&1
    /bin/modprobe sd_mod.o.gz > /dev/null 2>&1
    /bin/modprobe sr_mod.o.gz > /dev/null 2>&1
    /bin/modprobe sg.o.gz > /dev/null 2>&1
    /bin/modprobe libata.o.gz > /dev/null 2>&1

}

OLDDIR=`pwd`
cd /lib/modules/*/kernel/drivers/scsi

case $1 in
none)
    echo No scsi module tried
    ;;
all)
    load_common
    try_all_modules
    ;;
*)
    load_common
    try_and_leave $1
    ;;
esac

cd $OLDDIR
exit 0
