/* SmoothWall setup program.
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Lawrence Manning, 2001
 * Password stuff.
 * 
 * $Id: passwords.c,v 1.5.2.5 2007/08/12 19:09:52 gespinasse Exp $
 * 
 */

#include "setup.h"

extern FILE *flog;
extern char *mylog;

extern char **ctr;

extern int automode;

int getpassword(char *password, char *text);

/* Root password. */
int handlerootpassword(void)
{
	char password[STRING_SIZE];
	char commandstring[STRING_SIZE];
		
	/* Root password. */
	if (getpassword(password, ctr[TR_ENTER_ROOT_PASSWORD]) == 2)
		return 0;
	
	snprintf(commandstring, STRING_SIZE,
		"/bin/echo 'root:%s' | /usr/sbin/chpasswd",	password);
	if (runhiddencommandwithstatus(commandstring, ctr[TR_SETTING_ROOT_PASSWORD]))
	{
		errorbox(ctr[TR_PROBLEM_SETTING_ROOT_PASSWORD]);
		return 0;
	}
	
	return 1;
}

/* Backup password used to control backup.key export
   and crypt the key. */
int handlebackuppassword(void)
{
	char password[STRING_SIZE];
	char commandstring[STRING_SIZE];
	char message[1000];

	/* Backup password. */
	if (getpassword(password, ctr[TR_ENTER_BACKUP_PASSWORD]) == 2)
		return 0;
	snprintf(commandstring, STRING_SIZE,
		"/bin/echo 'backup:%s' | /usr/sbin/chpasswd",	password);
	if (runhiddencommandwithstatus(commandstring, ctr[TR_SETTING_BACKUP_PASSWORD]))
	{
		sprintf(message, ctr[TR_PROBLEM_SETTING_BACKUP_PASSWORD], NAME);
		errorbox(message);
		return 0;
	}
	
	return 1;
}

int handleadminpassword(void)
{
	char password[STRING_SIZE];
	char commandstring[STRING_SIZE];
	char message[1000];
		
	/* web interface admin password. */
	sprintf(message, ctr[TR_ENTER_ADMIN_PASSWORD], NAME, NAME);
	if (getpassword(password, message) == 2)
		return 0;
	snprintf(commandstring, STRING_SIZE,
		"/usr/bin/htpasswd -c -m -b " CONFIG_ROOT "/auth/users admin '%s'", password);
	sprintf(message, ctr[TR_SETTING_ADMIN_PASSWORD], NAME);
	if (runhiddencommandwithstatus(commandstring, message))
	{
		sprintf(message, ctr[TR_PROBLEM_SETTING_ADMIN_PASSWORD], NAME);
		errorbox(message);
		return 0;
	}

	return 1;
}

/* Taken from the cdrom one. */
int getpassword(char *password, char *text)
{
	char *values[] = {	NULL, NULL, NULL };	/* pointers for the values. */
	struct newtWinEntry entries[] =
	{ 
		{ ctr[TR_PASSWORD_PROMPT], &values[0], 2 },
		{ ctr[TR_AGAIN_PROMPT], &values[1], 2 },
		{ NULL, NULL, 0 }
	};
	int rc;
	int done;
	
	do
	{
		done = 1;
		newtPushHelpLine(ctr[TR_NO_PASSWORD_ECHO]);
		rc = newtWinEntries(get_title(), text,
			50, 5, 5, 20, entries, ctr[TR_OK], ctr[TR_CANCEL], NULL);
		newtPopHelpLine();

		if (rc != 2)
		{
			if (strlen(values[0]) == 0 || strlen(values[1]) == 0)
			{
				errorbox(ctr[TR_PASSWORD_CANNOT_BE_BLANK]);
				done = 0;
			}
			else if (strcmp(values[0], values[1]) != 0)
			{
				errorbox(ctr[TR_PASSWORDS_DO_NOT_MATCH]);
				done = 0;
			}
			else if (strchr(values[0], ' '))
			{
				errorbox(ctr[TR_PASSWORD_CANNOT_CONTAIN_SPACES]);
				done = 0;
			}
			else if (strlen(values[0]) < 6)
			{
				errorbox(ctr[TR_PASSWORD_MINIMAL_LENGTH_IS_6]);
				done = 0;
			}
			else if (strchr(values[0], '"') || strstr(values[0], "'"))
			{
				errorbox(ctr[TR_PASSWORD_CANNOT_CONTAIN_SIMPLE_OR_DOUBLE_QUOTE]);
				done = 0;
			}
			if (done ==0)
			{
				strcpy(values[0], "");
				strcpy(values[1], "");
			}
		}
	}
	while (!done);

	strncpy(password, values[0], STRING_SIZE);

	if (values[0]) free(values[0]);
	if (values[1]) free(values[1]);

	return rc;
}
