/* SmoothWall helper program - restartsquid
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Lawrence Manning, 2001
 * Restarting squid with transparent proxying.
 *
 * 05/02/2004 - Roy Walker <rwalker@miracomnetwork.com>
 * Exclude red network from transparent proxy to allow browsing to alias IPs
 * Read in VPN settings and exclude each VPN network from transparent proxy
 * 
 * $Id: restartsquid.c,v 1.7.2.13 2006/04/04 22:59:19 franck78 Exp $
 * 
 */
 
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "libsmooth.h"
#include "setuid.h"

char green_dev[STRING_SIZE] = "";
char blue_dev[STRING_SIZE] = "";

/* read the vpn config file and adds a rule for every net-to-net definition
    that skip the transparent rules REDIRECT
*/
void setdirectvpn(int setdirectvpn_green, int setdirectvpn_blue) {
	int count;
	char *result;
	char *name;
	char *type;
	char *running;
	char *vpn_network_mask;
	char *vpn_netaddress;
	char *vpn_netmask;
	FILE *file = NULL;
	char *conn_enabled;
	char buffer[STRING_SIZE];
	char s[STRING_SIZE];
	
	if (!setdirectvpn_green &&  !setdirectvpn_blue) 
		return; /* nothing to do */

	if (!(file = fopen(CONFIG_ROOT "/vpn/config", "r"))) {
		fprintf(stderr, "Couldn't open vpn config file");
		return; /* error! exit or return? */
	}
	/* WARNING: empty lines or comment not handled */
	while (fgets(s, STRING_SIZE, file) != NULL) {
		if (s[strlen(s) - 1] == '\n')
			s[strlen(s) - 1] = '\0';
		running = strdup (s);
		result = strsep(&running, ",");
		count = 0;
		name = NULL;
		type = NULL;
		vpn_network_mask = NULL;
		conn_enabled = NULL;
		while (result) {
			if (count == 1)
				conn_enabled = result;
			if (count == 2)
				name = result;
			if (count == 4)
				type = result;
			if (count == 12 )
				vpn_network_mask = result;
			count++;
			result = strsep(&running, ",");
		}

		if (strspn(name, LETTERS_NUMBERS) != strlen(name)) {
			fprintf(stderr, "Bad connection name: %s\n", name);
			continue;
		}
	
		if (! (strcmp(type, "net") == 0)) {
			continue;
		}

		/* Darren Critchley - new check to see if connection is enabled */
		if (! (strcmp(conn_enabled, "on") == 0)) {
			continue;
		}
	
		result = strsep(&vpn_network_mask, "/");
		count = 0;
		vpn_netaddress = NULL;
		vpn_netmask = NULL;
		while (result) {
			if (count == 0)
				vpn_netaddress = result;
			if (count == 1)
				vpn_netmask = result;
			count++;
			result = strsep(&vpn_network_mask, "/");
		}

		if (!VALID_IP(vpn_netaddress)) {
			fprintf(stderr, "Bad network for vpn connection %s: %s\n", name, vpn_netaddress);
			continue;
		}

		if ((!VALID_IP(vpn_netmask)) && (!VALID_SHORT_MASK(vpn_netmask))) {
			fprintf(stderr, "Bad mask for vpn connection %s: %s\n", name, vpn_netmask);
			continue;
		}

		memset(buffer, 0, STRING_SIZE);
		if (setdirectvpn_green) {
			if( snprintf(buffer, STRING_SIZE - 1,
					"/sbin/iptables -t nat -A SQUID -i %s -p tcp -d %s/%s --dport 80 -j RETURN",
					green_dev,
					vpn_netaddress,
					vpn_netmask) >= STRING_SIZE ) {
				fprintf(stderr, "Command too long\n");
				fclose(file);
				exit(1);
			}
			safe_system(buffer);
		}
		if (setdirectvpn_blue) {
			if (snprintf(buffer,
					STRING_SIZE - 1,
					"/sbin/iptables -t nat -A SQUID -i %s -p tcp -d %s/%s --dport 80 -j RETURN",
					blue_dev,
					vpn_netaddress,
					vpn_netmask) >= STRING_SIZE ) {
				fprintf(stderr, "Command too long\n");
				fclose(file);
				exit(1);
			}
			safe_system(buffer);
		}
	}
	fclose(file);
}
/*
	One or zero option can be specified, no parser !!!
options -f flush cache
	-r repair cache
	-t test, do not start squid if it was not running. It simplifies
	    calling from other tools.
*/
int main(int argc, char *argv[])
{
	int fd = -1;
	int enable = 0;
	int transparent = 0;
	int enable_blue = 0;
	int transparent_blue = 0;
	struct stat st;
	FILE *ipfile;
	char localip[STRING_SIZE] = "";
	struct keyvalue *net = NULL;
	struct keyvalue *squid = NULL;
	char buffer[STRING_SIZE];
	char proxy_port[STRING_SIZE];
	char red_netaddress[STRING_SIZE] = "";
	char red_netmask[STRING_SIZE] = "";
	char configtype[STRING_SIZE] = "";
	char redtype[STRING_SIZE] = "";
	char enableredvpn[STRING_SIZE] = "";
	char enablebluevpn[STRING_SIZE] = "";

	if (!(initsetuid()))
		exit(1);

	/* Retrieve the Squid pid file */
	if ( (fd = open("/var/run/squid.pid", O_RDONLY)) == -1 && 
			(argc >= 2) &&
			(strcmp(argv[1], "-t") == 0) ) {
		exit (0); /*Not running, no need to start with -t */
	}
	close(fd);

	/* Kill running squid */
	safe_system("/sbin/iptables -t nat -F SQUID");
	safe_system("/usr/sbin/squid -k shutdown >/dev/null 2>/dev/null");
	sleep(5);
	safe_system("/bin/killall -9 squid >/dev/null 2>/dev/null");
	
	/* See if proxy is enabled and / or transparent */
	if ((fd = open(CONFIG_ROOT "/proxy/enable", O_RDONLY)) != -1) {
		close(fd);
		enable = 1;
	}
	if ((fd = open(CONFIG_ROOT "/proxy/transparent", O_RDONLY)) != -1) {
		close(fd);
		transparent = 1;
	}
	if ((fd = open(CONFIG_ROOT "/proxy/enable_blue", O_RDONLY)) != -1) {
		close(fd);
		enable_blue = 1;
	}
	if ((fd = open(CONFIG_ROOT "/proxy/transparent_blue", O_RDONLY)) != -1) {
		close(fd);
		transparent_blue = 1;
	}

	/* Read the network configuration */
	net=initkeyvalues();
	if (!readkeyvalues(net, CONFIG_ROOT "/ethernet/settings")) {
		fprintf(stderr, "Cannot read ethernet settings\n");
		exit(1);
	}
	if (!findkey(net, "GREEN_DEV", green_dev)) {
		fprintf(stderr, "Cannot read GREEN_DEV\n");
		exit(1);
	}
	if (!findkey(net, "CONFIG_TYPE", configtype)) {
		fprintf(stderr, "Cannot read CONFIG_TYPE\n");
		exit(1);
	}

	findkey(net, "RED_TYPE", redtype);
	findkey(net, "RED_NETADDRESS", red_netaddress);
	findkey(net, "RED_NETMASK", red_netmask);
	findkey(net, "BLUE_DEV", blue_dev);
	freekeyvalues(net);

	/* Retrieve the RED ip address */
	stat(CONFIG_ROOT "/red/local-ipaddress", &st);
	if (S_ISREG(st.st_mode)) {
		if (!(ipfile = fopen(CONFIG_ROOT "/red/local-ipaddress", "r"))) {
			fprintf(stderr, "Couldn't open ip file\n");
			exit(0); 
		}
		if (fgets(localip, STRING_SIZE, ipfile)) {
			if (localip[strlen(localip) - 1] == '\n')
				localip[strlen(localip) - 1] = '\0';
		}
		fclose(ipfile);
		if (!VALID_IP(localip)) {
			fprintf(stderr, "Bad ip: %s\n", localip);
			exit(0);
		}
	}

	/* See if VPN software is enabled */
	net=initkeyvalues();
	if (!readkeyvalues(net, CONFIG_ROOT "/vpn/settings")) {
		fprintf(stderr, "Cannot read vpn settings\n");
		exit(1);
	}
	findkey(net, "ENABLED", enableredvpn);
	findkey(net, "ENABLED_BLUE", enablebluevpn);
	freekeyvalues(net);

	/* See if we need to flush/repair the cache */
	int saferestart = 0;
	if (argc >= 2) {
		struct passwd *pw;
		if((pw = getpwnam("squid"))) {
			/* flush */
			if (strcmp(argv[1], "-f") == 0) {
				endpwent(); /* probably paranoia, but just in case.. */
				unpriv_system("/bin/rm -rf /var/log/cache/*", pw->pw_uid, pw->pw_gid);
			}
			/* repair cache */
			if (strcmp(argv[1], "-r") == 0) {
				endpwent(); /* probably paranoia, but just in case.. */
				if (stat("/var/log/cache/swap.state", &st) == 0) {
					unpriv_system("/bin/rm -f /var/log/cache/swap.state", pw->pw_uid, pw->pw_gid);
				}
				saferestart = 1;
			}
		} else {
			endpwent();
		}
	}

	if (enable || enable_blue) {
		safe_system("/usr/sbin/squid -D -z"); 
		if (saferestart)
			safe_system("/usr/sbin/squid -DS");
		else
			safe_system("/usr/sbin/squid -D");
	}

	/* Retrieve the proxy port */
	if (transparent || transparent_blue) {
		squid=initkeyvalues();

		if (!readkeyvalues(squid, CONFIG_ROOT "/proxy/settings")) {
			fprintf(stderr, "Cannot read proxy settings\n");
			exit(1);
		}

		if (!(findkey(squid, "PROXY_PORT", proxy_port))) {
			strcpy (proxy_port, "800");
		} else {
			if(strspn(proxy_port, NUMBERS) != strlen(proxy_port)) {
				fprintf(stderr, "Invalid proxy port: %s, defaulting to 800\n", proxy_port);
				strcpy(proxy_port, "800");
			}
		}
		freekeyvalues(squid);
	}


	/* static (green/blue) interfaces must exist if transparence is requested */
	if (!VALID_DEVICE(green_dev) && transparent && enable) {
		fprintf(stderr, "Bad GREEN_DEV: %s\n", green_dev);
		exit(1);
	}

	if (! VALID_DEVICE(blue_dev) && transparent_blue && enable_blue) {
		fprintf(stderr, "Bad BLUE_DEV: %s\n", blue_dev);
		exit(1);
	}

	/* disable transparence for known vpn networks */
	setdirectvpn (enable && transparent && !strcmp(enableredvpn, "on"),
			enable_blue && transparent_blue && !strcmp(enablebluevpn, "on") );

	/* choose RED destination: 'localip' or 'red_netaddress/red_netmask' */
	char destination[STRING_SIZE] = "";
	if ( ( (strcmp(configtype, "2")==0) ||
			(strcmp(configtype, "3")==0) ||
			(strcmp(configtype, "6")==0) ||
			(strcmp(configtype, "7")==0) ) &&
			(VALID_IP(red_netaddress)) &&
			(VALID_IP(red_netmask)) && 
			(strcmp(redtype, "STATIC")==0) ) {
		snprintf (destination,STRING_SIZE - 1, "%s/%s", red_netaddress, red_netmask);
	} else {
		if (VALID_IP(localip)) {
			snprintf (destination,STRING_SIZE - 1, "%s", localip);
		}
	}

	/* RED may be down */
	if (!strlen(destination)) {
		fprintf(stderr, "Cannot determine RED network.\n");
	}

	/* install the transparency rules */
	/* green transparent ? */
	if (transparent && enable) {
		/* direct http GREEN-->RED network */
		memset(buffer, 0, STRING_SIZE);
		if ( snprintf(buffer, STRING_SIZE - 1,
				"/sbin/iptables -t nat -A SQUID -i %s -p tcp -d %s --dport 80 -j RETURN",
				green_dev,
				destination) >= STRING_SIZE ) {
			fprintf(stderr, "Command too long\n");
			exit(1);
		}
		if (strlen(destination)) safe_system(buffer);  /* only id known RED */

		/* install the redirect for other port http destinations from green */
		memset(buffer, 0, STRING_SIZE);
		if( snprintf(buffer, STRING_SIZE - 1,
				"/sbin/iptables -t nat -A SQUID -i %s -p tcp --dport 80 -j REDIRECT --to-port %s",
				green_dev,
				proxy_port) >= STRING_SIZE ) {
			fprintf(stderr, "Command too long\n");
			exit(1);
		}
		safe_system(buffer);
	}
	/* blue transparent ? */
	if (transparent_blue && enable_blue) {
		/* direct http BLUE-->RED network */
		memset(buffer, 0, STRING_SIZE);
		if( snprintf(buffer, STRING_SIZE - 1,
				"/sbin/iptables -t nat -A SQUID -i %s -p tcp -d %s --dport 80 -j RETURN",
				blue_dev,
				destination) >= STRING_SIZE ) {
			fprintf(stderr, "Command too long\n");
			exit(1);
		}
		if (strlen(destination)) safe_system(buffer);  /* only id known RED */

		/* install the redirect for other port http destinations from blue */
		memset(buffer, 0, STRING_SIZE);
		if( snprintf(buffer, STRING_SIZE - 1,
				"/sbin/iptables -t nat -A SQUID -i %s -p tcp --dport 80 -j REDIRECT --to-port %s",
				blue_dev,
				proxy_port) >= STRING_SIZE ) {
			fprintf(stderr, "Command too long\n");
			exit(1);
		}
		safe_system(buffer);
	}
	return 0;
}
