/* SmoothWall helper program - smoothiedeath
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Gilles Espinasse, 2007-06-03
 * uid/gid parts derivated from logwatch.c (c) 2003-07-12 Robert Kerr
 *
 * Helper to load and merge snort rules from GUI
 *
 * $Id: snortrules.c,v 1.1.2.3 2007/06/10 18:19:45 gespinasse Exp $
 *
 */

#include <stdio.h>		/* perror stderr */
#include <string.h>		/* strcmp memset strlen */
#include <unistd.h>		/* close unlink setgid setuid sleep */
#include <fcntl.h>		/* open O_WRONLY O_CREAT O_EXCL */
#include <sys/stat.h>		/* S_IWUSR */
#include <pwd.h>		/* getpwnam endpwent*/
#include <grp.h>		/* setgroups */
#include "setuid.h"

#define LOCKFILE	"/var/log/snort/snortrules.lock"
#define LOGFILE		"/var/log/snort/lastrulesupdate"
#define RULESTARGZ	"/var/log/snort/rules.tar.gz"
#define OINKMASTERCONF	"/var/ipcop/snort/oinkmaster.conf"

char tmptargzfile[STRING_SIZE];		/* loaded file from snort.org */
char lockfile[STRING_SIZE];		/* prevent using snortrules more than once at the same time */
int plockfd;

void exithandler(void)
{
	/* remove lock */
	if (plockfd) close(plockfd);
	if (unlink(LOCKFILE)) perror("snortrules : fail to remove " LOCKFILE);
}

int main(int argc, char *argv[])
{
	int i;
	int md5=0, load=0, merge=0, verbose=0;
	struct passwd *pw;
	gid_t groups[2];

	if (!(initsetuid()))
		exit(1);

	for (i=0; i<argc; i++) {
		if (!strcmp(argv[i], "--md5")) md5=1;
		if (!strcmp(argv[i], "--load")) load=1;
		if (!strcmp(argv[i], "--merge")) merge=1;
		if (!strcmp(argv[i], "--verbose")) verbose = 1;
	}

	/* lookup snort user */
	if(!(pw = getpwnam("snort"))) {
		fprintf(stderr,"Couldn't find snort user.\n");
		exit(1);
	}
	/* paranoia... */
	memset(pw->pw_passwd, 0, strlen(pw->pw_passwd));
	endpwent();
	/* more paranoia */
	if(!pw->pw_uid || !pw->pw_gid) {
		fprintf(stderr,"snort user appears to be UID or GID 0, aborting.\n");
		exit(1);
	}

	/* drop privs */
	groups[0] = groups[1] = pw->pw_gid;
	if (setgroups(1,groups)) { perror("Couldn't clear group list"); exit(1); }
	if (setgid(pw->pw_gid))  { perror("Couldn't setgid(snort)"); exit(1); }
	if (setuid(pw->pw_uid))  { perror("Couldn't setuid(snort)"); exit(1); }

	/* not any recognized parameter */
	if (md5+load+merge==0) {
		fprintf(stderr,"Usage:snortrules [--md5|--load|--merge]\n");
		fprintf(stderr,"--md5\tLoad rules md5 from snort.org to display on GUI if a new package is available\n");
		fprintf(stderr,"--load\tLoad rules set to /var/log/snort\n");
		fprintf(stderr,"--merge\tMerge new rules set to existing rules in use\n");
		exit(2);
	}

	/* avoid oinkmaster error : could not change directory back to /root */
	chdir("/var/log/snort");

	/* only one instance of snortrules should run at the same time */
	/* create a lock file */
	if((plockfd = open(LOCKFILE, O_WRONLY | O_CREAT | O_EXCL, S_IWUSR ))==-1) {
		perror("snortrules fail to create " LOCKFILE);
		exit(3);
	}

	/* now exithandler will have something to do */ 
	atexit(exithandler);

	if (md5) {
		if (safe_system("/usr/local/bin/snortrules.pl md5")) {
			exit(4);
		} else {
			exit(0);
		}
	}

	if (load) {
		/* todo check free space on /var/log/snort */
		if (safe_system("/usr/local/bin/snortrules.pl load")) {
			fprintf(stderr, "snortrules : load error\n");
			exit(5);
		}
	}

	if (merge) {
		/* todo
		 * check free space on /etc/snort for file merging
		 * check logfile
		 */
		
		/* redirect sdtout and stderr to logfile */
		if (safe_system ("/usr/local/bin/oinkmaster.pl -s -u file://" RULESTARGZ
				" -C " OINKMASTERCONF " -o /etc/snort/rules &>" LOGFILE)) {
			fprintf(stderr, "snortrules : oinkmaster error\n");
			exit(6);
		}
	}
	exit(0);
}
