#!/usr/bin/perl
#
# This code is distributed under the terms of the GPL
#
# (c) IPCop Team
#
# $Id: snortrules.pl,v 1.1.2.4 2007/10/20 07:56:31 gespinasse Exp $
#

use LWP::UserAgent;
use File::Copy;
use File::Temp qw/ tempfile /;
use strict;

# enable only the following on debugging purpose
#use warnings; no warnings 'once';# 'redefine', 'uninitialized';

require 'CONFIG_ROOT/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";

# early define sub with prototypes;
sub getmd5($);
sub updaterules($);
sub geturl($);
sub tagisold($ $);

my %snortsettings=();
my %snorttags=(); # separate from settings to not been include in backup/restore
my %netsettings=();
our $md5 = '';
&General::readhash("${General::swroot}/ethernet/settings", \%netsettings);

# installed rules
# only one tag for (subscripted|registered) as we update only one at a time
# if we update both, we will not know if the other is more recent or older
$snorttags{'VRTMERGEDATE'} = '';	# last time oinkmaster merge rules
 # reading md5 of rules file is allowed only once every 15 mn so we need to track when was the last time to avoid error
$snorttags{'MD5PROBEVALUE'} = '';	# file md5 value
$snorttags{'MD5PROBEDATE'} = 0;		# last md5 file read date
$snorttags{'VRTLOADMD5'} = '';		# last loaded VRT file md5
$snorttags{'VRTLOADDATE'} = 0;		# last VRT file load date

&General::readhash('/etc/snort/rulestags', \%snorttags);
# feedback
$snorttags{'MESSAGE'} = ''; # always start with empty message
my $logfile = '/var/log/snort/lastrulesupdate';
my $rulestargz = '/var/log/snort/rules.tar.gz';

$snortsettings{'RULESTYPE'} = 'nothing';
$snortsettings{'OINKCODE'} = '';
&General::readhash("${General::swroot}/snort/settings", \%snortsettings);

my $rulesbranch="2.6"; # version should match snort branch version
my $VRTurl="http://www.snort.org/pub-bin/oinkmaster.cgi/$snortsettings{'OINKCODE'}/snortrules-snapshot";
my %urls= ('subscripted'  => "$VRTurl-${rulesbranch}_s.tar.gz",
	  'registered' => "$VRTurl-${rulesbranch}.tar.gz");

( my $prog = $0 ) =~ s!^.*/!!; #prog name without path
# Simply die for cases that the web interface does not allow
unless (defined($ARGV[0])) {
	die "Usage
	$prog md5 : read selected rules md5
	$prog load : load selected rules set to /var/log/snort\n";
}

# could be started from snortrules helper only or rights on created files will differ
# if not snort uid
if ($> !=100) { die "Error : don't directly use $prog but /usr/local/bin/snortrules helper only\n"; }

unless ($ARGV[0] =~ /^(md5|load)$/) { die "$prog invalid parameter : only md5 or load is allowed\n"; }

unless ($snortsettings{'RULESTYPE'} =~ /^(nothing|registered|subscripted)$/) { die "Error : RULESTYPE invalid\n"; }
if ( $snortsettings{'RULESTYPE'} eq 'nothing' ) { die "Error : not any rules set selected\n"; }

if (! -e "${General::swroot}/red/active" ) { die "Error : not connected. Can't reach snort.org\n"; }
if ( $snortsettings{'RULESTYPE'} =~ /^(registered|subscripted)$/ ) {
	if ($snortsettings{'OINKCODE'} eq '' ) { die "Error : OINKCODE is empty for VRT rules\n"; }
	if ($snortsettings{'OINKCODE'} !~ /^[a-z0-9]+$/) { die "Error : bad OINKCODE content\n"; }
}

# read selected md5
if ($ARGV[0] eq 'md5') {
	if ( $snortsettings{'RULESTYPE'} =~ /^(registered|subscripted)$/) {
		$snorttags{'MD5PROBEVALUE'}=&getmd5($urls{$snortsettings{'RULESTYPE'}});
		unless (defined $snorttags{'MD5PROBEVALUE'}) {
			# add with wich setting the problem occure
			$snorttags{'MESSAGE'} .= " $snortsettings{'RULESTYPE'} md5";
		}
	}
}

# load selected rules
if ($ARGV[0] eq 'load') {
	if ( $snortsettings{'RULESTYPE'} =~ /^(registered|subscripted)$/ ) {
		&updaterules($urls{$snortsettings{'RULESTYPE'}});
	}
}
# allow multiples lines separated by <br />
if ( $snorttags{'MESSAGE'} ne '' ) {
	$snorttags{'MESSAGE'} = '<pre>'.$snorttags{'MESSAGE'}.'</pre>';
}
&General::writehash("/etc/snort/rulestags", \%snorttags);

# return undef and an error message in $snorttags{'MESSAGE'} in case of error
# return md5 if success
sub getmd5($)
{
	# Retrieve MD5 sum from $url.md5 file
	my $url=shift;
	if (&tagisold($snorttags{'MD5PROBEDATE'},$snorttags{'MD5PROBEVALUE'})) {
		my $md5buf = &geturl("$url.md5");
		return undef unless $md5buf;
		if (0) { # 1 to debug
			my $filename='';
			my $fh='';
			($fh, $filename) = tempfile('/tmp/XXXXXXXX',SUFFIX => '.md5' );
			binmode ($fh);
			syswrite ($fh, $md5buf->content);
			close($fh);
		}
		my $md5 = $md5buf->content;
		$snorttags{'MD5PROBEDATE'} = time(); # number of second
		return substr($md5,0,32);	#md5 is 32 chars (remove trailing spaces or newline)
	} else {
		# keep the same
		return $snorttags{'MD5PROBEVALUE'};
	}
}

# return undef and an error message in $snorttags{'MESSAGE'} in case of error
# return 0 if success
sub updaterules($)
{
	my $url=shift;
	my $error; # used to delay return on oinkmaster error until redirection restored

	# retrieve file name from $url to be displayed
	my @parts = split(/\//, $url); # name is $parts[-1]

	$md5 = &getmd5($url);
	return undef unless defined $md5;

	if (!&tagisold($snorttags{'VRTLOADDATE'},$snorttags{'VRTLOADMD5'})) {
		# keep the same file as we can't upload before 15 mn
		return 0;
	}
	# read the rules file
	# get a ref to a datastream
	my $databuf = &geturl($url);
	return undef unless $databuf;
	if (index($databuf->content, "\037\213") == -1 ) { # \037\213 is .gz beginning
		$snorttags{'MESSAGE'} .= $Lang::tr{'invalid loaded file'};
		return undef;
	}
	# and write the datastream to a file
	open(FILE, ">$rulestargz");
	binmode (FILE);
	syswrite (FILE, $databuf->content);
	close(FILE);

	# Check md5sum
	my $realmd5 = `/usr/bin/md5sum $rulestargz`;
	chop ($realmd5);
	$realmd5 =~ s/^(\w+)\s.*$/$1/;
	if ($md5 ne $realmd5) {
		$snorttags{'MESSAGE'} .= "$Lang::tr{'invalid md5sum'} $parts[-1]";
		return undef;
	}
	$snorttags{'VRTLOADMD5'} = $realmd5;
	$snorttags{'VRTLOADDATE'} = time(); # number of second;
	return 0; # ok
}

sub geturl($)
{
	my $url=$_[0];
	my $downloader = LWP::UserAgent->new;
	$downloader->timeout(30);
	my %proxysettings=();
	&General::readhash("${General::swroot}/proxy/settings", \%proxysettings);
	if ($_=$proxysettings{'UPSTREAM_PROXY'}) {
		my ($peer, $peerport) = (/^(?:[a-zA-Z ]+\:\/\/)?(?:[A-Za-z0-9\_\.\-]*?(?:\:[A-Za-z0-9\_\.\-]*?)?\@)?([a-zA-Z0-9\.\_\-]*?)(?:\:([0-9]{1,5}))?(?:\/.*?)?$/);
		if ($proxysettings{'UPSTREAM_USER'}) {
			$downloader->proxy("http","http://$proxysettings{'UPSTREAM_USER'}:$proxysettings{'UPSTREAM_PASSWORD'}@"."$peer:$peerport/");
		} else {
			$downloader->proxy("http","http://$peer:$peerport/");
		}
	}
	my $databuf = $downloader->get($url,'Cache-Control','no-cache');
	if ($databuf->code == 200) {
		return $databuf;
	} elsif ($databuf->code == 403) {	#specifics messages : time to wait before allowed update
		$databuf->content =~ /<title>(.*)<\/title>/;
		$snorttags{'MESSAGE'} .= "$1";
	} elsif ($databuf->code == 500) {
		$snorttags{'MESSAGE'} .= $Lang::tr{'error 500'};
	} else {				#unknown message
		$snorttags{'MESSAGE'} .= "$Lang::tr{'bad return code'}: $databuf->code";
		General::log('snort',"$Lang::tr{'bad return code'}: $databuf->code");
	}
	return undef;
}

sub tagisold($ $)
{
	# could only read md5 or rules file once every 15 mn
	my $tagdate=shift;
	my $tagmd5 =shift;

	if ( $tagdate == 0 || $tagmd5 eq '' ) {
		return 1;
	} else {
		# are we 60*15 s after last read
		if ( ($tagdate+900) < time() ) {
			return 1;
		}
	}
	return 0;
}
