#!/bin/bash
#
# This file is part of the IPCop Firewall.
#
# IPCop is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# IPCop is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with IPCop; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# (c) 2008, the IPCop team
#
# Achim Weber 2008-07-04
#		Detect running kernel flavor (single vs. SMP, ACPI on vs. ACPI off)
#		and select the most recent kernel in grub as default at next boot
# 		inspired by http://lists.opensuse.org/opensuse-mobile-de/2005-06/msg00052.html
#
# $Id: updateGrubDefault.sh,v 1.1.2.2 2008/07/22 10:32:02 gespinasse Exp $

# acpi=ht is no more set since 1.4.20 as this a source of bug with some bioses

ACPI_OFF=`/bin/grep 'acpi=off' /proc/cmdline`
SMP=`/bin/grep '\-smp ' /proc/version`

# we can't trust totally kernelsettings KERNEL value
# it is restored on backup and may not match actual usage, so do it differently
if [[ -f /boot/vmlinuz-smp && /boot/vmlinuz ]]; then
	KERNEL=BOTH
else
	KERNEL=ONE
fi

if [ "$ACPI_OFF" != "" ]; then
	MSG='selected kernel acpci=off'
	if [ "$KERNEL" = "ONE" ]; then
		DEFAULT=0
	elif [ "$SMP" = "" ]; then
		DEFAULT=0
	else
		MSG="$MSG smp"
		DEFAULT=1
	fi
elif [ "$KERNEL" = "ONE" ]; then
	MSG='Selected kernel acpci'
	DEFAULT=1
elif [ "$SMP" = "" ]; then
	MSG='Selected kernel acpci single'
	DEFAULT=2
else
	MSG='Selected kernel acpci smp'
	DEFAULT=3
fi

/bin/echo "$MSG ,line $DEFAULT"
/bin/echo "savedefault --default=$DEFAULT " | /usr/sbin/grub --no-curses --device-map=/boot/grub/device.map >/dev/null
