#!/bin/bash

echo "prepare pci and usb maps for publication on the wiki"
if [ ! -s build/lib/modules ]; then
	echo "use this tool at top level after full compilation with ./tools/prepareIDmap.sh"
	exit 1
fi

KVER=`grep --max-count=1 VER lfs/linux | awk '{ print $3 }'`

# suppress first line temporary and sort by module name
sed -e '/module/d' build/lib/modules/$KVER/modules.pcimap | sort > doc/tmpmap1.txt

# cells are seperated by ;
# CnxADSL is interpreted as link, fix that
sed -i	-e 's/  */ /g'		\
	-e 's/ /;/g'		\
	-e 's/^/;/'		\
	-e 's/|#|/;/'		\
	-e 's/CnxADSL/!CnxADSL/'		\
  doc/tmpmap1.txt

# get rid of LF character
tr -d "\n" < doc/tmpmap1.txt > doc/tmpmap2.txt

# add comment
echo "PCI id from $KVER compilation `date +'%Y-%m-%d'`" > doc/PCIidmap.txt
echo " "  >>	doc/PCIidmap.txt

# add table header
echo -n "{{table columns=\"8\" cells=\"==pci module==;==vendor==;==device==;==subvendor==;==subdevice==;==class==;==class_mask==;==driver_data==" \
  >>	doc/PCIidmap.txt

cat doc/tmpmap2.txt >> doc/PCIidmap.txt

# and table terminator
echo -n "\"}}" >> doc/PCIidmap.txt


# suppress first line temporary and sort by module name
sed	-e '/module/d' build/lib/modules/$KVER/modules.usbmap | sort > doc/tmpmap1.txt

# cells are seperated by ;
sed -i	-e 's/  */ /g'		\
	-e 's/ /;/g'		\
	-e 's/^/;/'		\
	-e 's/|#|/;/'		\
	doc/tmpmap1.txt

# get rid of LF character
tr -d "\n" < doc/tmpmap1.txt > doc/tmpmap2.txt

# add comment
echo "USB id from $KVER compilation `date +'%Y-%m-%d'`" > doc/USBidmap.txt
echo " "  >>	doc/USBidmap.txt

# add table header
echo -n "{{table columns=\"13\" cells=\"==usb module==;==match_flags==;==idVendor==;==idProduct==;==bcdDevice_lo==;==bcdDevice_hi==;==b!DeviceClass==;==b!DeviceSubClass==;==b!DeviceProtocol==;==b!InterfaceClass==;==b!InterfaceSubClass==;==b!InterfaceProtocol==;==driver_info==" \
  >>	doc/USBidmap.txt

cat doc/tmpmap2.txt >> doc/USBidmap.txt

# and table terminator
echo -n "\"}}" >> doc/USBidmap.txt

# clean up
rm doc/tmpmap1.txt
rm doc/tmpmap2.txt

echo "Result send in doc directory"
