/* Derivated from SmoothWall helper programs
 *
 * This program is distributed under the terms of the GNU General Public
 * Licence.  See the file COPYING for details.
 *
 * (c) Daniel Goscomb, 2001
 *
 * Modifications and improvements by Lawrence Manning.
 *
 * 19/04/03 Robert Kerr Fixed root exploit
 *
 * 20/08/05 Achim Weber 20 Modified to have a binary for the new firewall options page in IPCop 1.4.8
 *
 * 02/10/05 Gilles Espinasse treat only ping actually
 *
 * $Id: setfilters.c,v 1.1.2.4 2007/06/15 17:14:31 franck78 Exp $
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "libsmooth.h"
#include "setuid.h"

struct keyvalue *kv = NULL;
FILE *ifacefile = NULL;

void exithandler(void)
{
	if(kv)
		freekeyvalues(kv);
}

void setping() {
	char disableping[STRING_SIZE];

	/* default is not disable ping */
	if (!findkey(kv, "DISABLEPING", disableping)) {
		fprintf(stderr, "No DISABLEPING value: ping response enabled\n");
		strcpy (disableping,"NO");
	}

	if (strcmp(disableping, "NO") != 0 && strcmp(disableping, "ONLYRED") != 0 && strcmp(disableping, "ALL") != 0) {
		fprintf(stderr, "Unknown DISABLEPING value: %s; ping response enabled\n", disableping);
		strcpy(disableping,"NO");
	}

	/* Get RED */
	char iface[STRING_SIZE] = "";
	int redAvailable = 0;
	if ((ifacefile = fopen(CONFIG_ROOT "/red/iface", "r"))) {
		if (fgets(iface, STRING_SIZE, ifacefile)) {
			if (iface[strlen(iface) - 1] == '\n')
				iface[strlen(iface) - 1] = '\0';
		}
		fclose (ifacefile);
		if (!VALID_DEVICE(iface)) {
			fprintf(stderr, "Bad iface: %s\n", iface);
			return;
		}
		redAvailable = 1;
	}

	/* don't need to do anything if ping is disabled, so treat only other cases */
	if (strcmp(disableping, "NO") == 0
		|| (strcmp(disableping, "ONLYRED") == 0 && redAvailable == 0)) {
		// We allow ping (icmp type 8) on every interfaces
		// or RED is not available, so we can enable it on all (available) Interfaces
		safe_system("/sbin/iptables -A GUIINPUT -p icmp --icmp-type 8 -j ACCEPT");
	} else {
		// Allow ping only on internal interfaces
		if(strcmp(disableping, "ONLYRED") == 0) {
			char command[STRING_SIZE];
			memset(command, 0, STRING_SIZE);
			snprintf(command, STRING_SIZE - 1,
				"/sbin/iptables -A GUIINPUT -i ! %s  -p icmp --icmp-type 8 -j ACCEPT", iface);
			safe_system(command);
		}
	}
}

int main(void)
{

	if (!(initsetuid()))
		exit(1);

	atexit(exithandler);

	/* reset all */
	safe_system("/sbin/iptables -F GUIINPUT");

	/* Read in and verify config */
	kv=initkeyvalues();

	if (!readkeyvalues(kv, CONFIG_ROOT "/optionsfw/settings")) {
		fprintf(stderr, "Cannot read firewall option settings\n");
		exit(1);
	}

	/* setup individual tunning */
	setping();

	//setgui
	freekeyvalues(kv);
	kv = NULL;

	return 0;
}
